package auxiliary;

import java.lang.Integer;
import java.lang.NumberFormatException;

/*** Command Line Reader ***/

/*** Accepts format: tazup [-g] [port] ***/
/*** Accepts format: tazup -h          ***/
/*** WHERE: -g is for GUI              ***/
/***        port is implicitly 4747    ***/
/***        -h is for help             ***/

public class CommandLineReader {
        private int port;
        private boolean gui;
        
        public CommandLineReader() {
                port = 4747;
                gui = false;
        }
        
        public boolean readCommandLine(String args[]) {
                if (args.length == 1 && args[0].equals("-h"))
                        return false;
            
                if (args.length > 2)
                        return false;
                
                String portArg = null;
                
                if (args.length == 2) {
                    if (args[0].equals("-g"))
                            portArg = args[1];
                    else if (args[1].equals("-g"))
                            portArg = args[0];
                    else return false;
                    
                    gui = true;
                }
                
                if (args.length == 1) {
                        if (args[0].equals("-g"))
                                gui = true;
                        else
                                portArg = args[0];
                }
                
                if (portArg == null)
                        return true;
                
                try {
                        Integer portNo = new Integer(portArg);
                        if (portNo < 1 || portNo > 65535) return false;
                        port = portNo;
                } catch (NumberFormatException E) {
                        return false;
                }

                return true;
        }
        
        public boolean gui() {
                return gui;
        }
        
        public int port() {
                return port;
        }
        
        public static void printUsage() {
                System.out.println("");
                System.out.println("TazUP P2P filesharing system");
                System.out.println("    2011 (C) Peter Csiba, Martin Šrámek, Jakub Vaňo");
                System.out.println("");
                System.out.println("USAGE: tazup [-g] [port]");
                System.out.println("       tazup -h");
                System.out.println("");
                System.out.println("DESCRIPTION:");
                System.out.println("    -g    Makes tazup use GUI (the default is CUI).");
                System.out.println("");
                System.out.println("    port  The port number used for communication.");
                System.out.println("          Must be between 1 and 65535. If not set");
                System.out.println("          explicitly, 4747 is used as the default.");
                System.out.println("");
                System.out.println("    -h    Show this information screen.");
                System.out.println("");
        }
}
