package auxiliary;

public class Convert {
	public static byte[] intToBytes(int x) {
		byte[] b = new byte[4];
        
        for (int i = 0; i < 4; i++) {
            int offset = (3 - i) * 8;
            b[i] = (byte) ((x >> offset) & 0xFF);
        }
        
        return b;
	}
	
	public static int bytesToInt(byte[] bytes, int offset) {
		int id = 0;
        for (int i=3; i>=0; i--) {
        	int shift = (3-i)*8;
        	int temp = (int)bytes[i+offset];
        	if (temp < 0) temp = ((-temp)^0xff) +1;
        	id |= temp << shift;
        }
        return id;
	}
	
	//!Copy paste!
	public static byte[] longToBytes(long x) {
		byte[] b = new byte[8];
        
        for (int i = 0; i < 8; i++) {
            int offset = (7 - i) * 8;
            b[i] = (byte) ((x >> offset) & 0xFF);
        }
        
        return b;
	}
	
	public static long bytesToLong(byte[] bytes, int offset) {
		long id = 0;
        for (int i=7; i>=0; i--) {
        	int shift = (7-i)*8;
        	long temp = (long)bytes[i+offset];
        	if (temp < 0) temp = ((-temp)^0xff) +1;
        	id |= temp << shift;
        }
        return id;
	}
	//
	public static byte[] shortToBytes(short x) {
		byte[] b = new byte[2];
        
        for (int i = 0; i < 2; i++) {
            int offset = (1 - i) * 8;
            b[i] = (byte) ((x >> offset) & 0xFF);
        }
        
        return b;
	}
	
	public static short bytesToShort(byte[] bytes, int offset) {
		short id = 0;
        for (int i=1; i>=0; i--) {
        	int shift = (1-i)*8;
        	int temp = (int)bytes[i+offset];
        	if (temp < 0) temp = ((-temp)^0xff) +1;
        	id |= temp << shift;
        }
        return id;
	}
	
	//Mainly used in SHA1
	public static byte alphaHexToByte(char c1){
		//0123456789abcdef -- these characters are not adjacent in ASCII
		if(c1 >= 'a' && c1 <= 'f')
			return (byte)(10 + c1 - 'a');
		return (byte)(c1 - '0');
	}
	public static byte alphaHexToByte(char c1, char c2){
		return (byte) (Convert.alphaHexToByte(c1) * (byte)(16) + Convert.alphaHexToByte(c2));
	}
	public static byte alphaHexToByte(String s){
		return Convert.alphaHexToByte(s.charAt(0), s.charAt(1));
	}
	public static String byteToAlphaHex(byte b){
		return Integer.toString((b & 0xff) + 0x100, 16).substring(1);
	}
	//If the string has odd length, then the inverse function (bytesToAlphaHexString) will return s + '0'.
	public static byte[] alphaHexStringToBytes(String s){
		return Convert.alphaHexStringToBytes(s, -1);
	}
	public static byte[] alphaHexStringToBytes(String s, int length){
		if(length < 0)
			length = (s.length() + 1) / 2;
		
		byte[] result = new byte[length];
		for(int i=0; i<length; i++){
			result[i] = Convert.alphaHexToByte(
				(2*i < s.length()) ? s.charAt(2*i) : '0',
				(2*i+1 < s.length()) ? s.charAt(2*i+1) : '0'
			);
		}
		return result;
	}
	//Always returns String of even length. 
	public static String bytesToAlphaHexString(byte[] data){
		//--http://www.mkyong.com/java/how-to-generate-a-file-checksum-value-in-java/
	    StringBuffer sb = new StringBuffer("");
	    for (int i = 0; i < data.length; i++) {
	    	sb.append(Convert.byteToAlphaHex(data[i]));
	    }
	    return sb.toString();
	}
}
