package auxiliary;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.lang.Comparable;
import java.lang.ClassCastException;

public class SHA1 implements Comparable<Object> {
	static public int SHA1_LENGTH = 20;
	
	private byte[] value;
	
	public SHA1() {
		value = new byte[SHA1_LENGTH];
		for (int i=0; i<SHA1_LENGTH; i++) value[i] = 0;
	}
	
	public SHA1(byte[] value) {
		if (value.length != SHA1_LENGTH)
			throw new IllegalArgumentException();
			
		this.value = value;
	}
	
    //convert the bytes to hex format
	public String toString(){
        return Convert.bytesToAlphaHexString(this.value);
	}

	//Creates SHA1 from HexString (a,b,c) -- inverse for toString() method
	public static SHA1 getHashFromHex(String s){
		return new SHA1(Convert.alphaHexStringToBytes(s, SHA1_LENGTH));
	}
	
	public static SHA1 getHashOfString(String s){
		// NoSuchAlgorithm Exception
        MessageDigest md;
        //!TODO -- check somewhere, if SHA1 exists on this machine. 
		try {
			md = MessageDigest.getInstance("SHA1");
		} catch (NoSuchAlgorithmException e) { 
			//Total fail
			return null;
		}
		md.update(s.getBytes());
		
		return new SHA1(md.digest());
	}
	
	//Creates SHA hash from a file located at filepath. 
	//--returns null if not successful
	public static SHA1 getFileHash(String filepath) {
		//CODE COPIED FROM (Petrz added some error handling):
        //--http://www.mkyong.com/java/how-to-generate-a-file-checksum-value-in-java/
    	
		// NoSuchAlgorithm Exception
        MessageDigest md;
        //!TODO -- check somewhere, if SHA1 exists on this machine. 
		try {
			md = MessageDigest.getInstance("SHA1");
		} catch (NoSuchAlgorithmException e) { 
			//Total fail
			return null;
		}
        // NoSuchFile Exception
        FileInputStream fis = null;
		try {
			fis = new FileInputStream(filepath);
		} catch (FileNotFoundException e) {
			//No such a file
			//!TODO - error handling
			e.printStackTrace();
			return null;
		}
        
        byte[] dataBytes = new byte[1024];
     
        int nread = 0; 
        // IOException
        try {
			while ((nread = fis.read(dataBytes)) != -1) {
			  md.update(dataBytes, 0, nread);
			}
		} 
        //If not read, just break;
        catch (IOException e) {} 
		finally{
			//!TODO - handle unsuccessful file closure ?
			try {
				fis.close();
			} catch (IOException e) {}
		}
     
        return new SHA1(md.digest());
	}
	
	public byte[] value() {
		return value.clone();
	}
	
	@Override
	public boolean equals(Object obj) {
		if (! (obj instanceof SHA1)) return false;
		SHA1 sha1 = (SHA1) obj;
		if (this.value.length != sha1.value.length) return false;
		for (int i=0; i<this.value.length; i++)
			if (this.value[i] != sha1.value[i]) return false;
		return true;
	}
	
	@Override
	public int hashCode() {
		int result = 0;

        for (int i = 0; i < value.length - 3; i+=4) {
        	result ^= (value[i] << 24) 
        			+ (value[i+1] << 16) 
        			+ (value[i+2] << 8) 
        			+ (value[i+3]);
        }
        
        return result;
	}
        
        @Override
        public int compareTo(Object obj) {
                if (! (obj instanceof SHA1))
                        throw new ClassCastException();
                
                byte[] their = ((SHA1) obj).value();
                        
                for (int i = 0; i < SHA1_LENGTH; i++) {
                        if (value[i] < their[i]) return -i - 1;
                        if (value[i] > their[i]) return  i + 1;
                }
                
                return 0;
        }
}
