package gui;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JProgressBar;
import javax.swing.JTextField;
import javax.swing.JLabel;
import javax.swing.JTable;
import javax.swing.table.TableModel;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.JTableHeader;
import javax.swing.border.LineBorder;
import javax.swing.JScrollPane;
import javax.swing.JOptionPane;
import java.awt.Color;
import java.awt.Dimension;
import java.math.BigInteger;

import auxiliary.*;
import modules.*;
import types.*;

public class AppWindow extends JFrame implements SearchResultReceiver, ProgressSignalReceiver {

	/**
	 * 
	 */
	private static final long serialVersionUID = 4245246633629876362L;
	private Thread thisThread;
        private final JTable resultsTable;
        private final JProgressBar progressBar;
        private DataStorage data;
        private SearchResult[] currentResults;
        private JFrame thisFrame;
        
        public void receiveSearchMessage() {
                currentResults = data.getSearchResultsSortedByTime();
                
                TableModel dataModel = new AbstractTableModel() {
                        public int getColumnCount() {
                                return 4;
                        }
                        
                        @Override
                        public String getColumnName(int column) {
                                switch (column) {
                                        case 0: return "ID";
                                        case 1: return "Filename";
                                        case 2: return "Size";
                                        case 3: return "Hash";
                                        default: return null;
                                }
                        }
                                
                        public int getRowCount() {
                                return currentResults.length;
                        }
                        
                        public Object getValueAt(int row, int col) {
                                switch (col) {
                                        case 0: return new Integer(currentResults[row].peer().value());
                                        case 1: return currentResults[row].filename();
                                        case 2: return new BigInteger(Long.toString(currentResults[row].filesize()));
                                        case 3: return currentResults[row].checksum().toString();
                                        default: return null;
                                }
                        }
                };
                
                resultsTable.setModel(dataModel);
        }
        
        public void receiveProgressSignal(double progress) {
                int value = new Double(progress * 1000).intValue();
                if (value < 0) value = 0;
                if (value > 1000) value = 1000;
                progressBar.setValue(value);
                
                if (value == 1000) {
                        JOptionPane.showMessageDialog(null, "Download completed.");
                }
        }
        
        public Thread thisThread() {
                return thisThread;
        }

	public AppWindow(){
		thisThread = Thread.currentThread();
		thisFrame = this;
		data = DataStorage.getInstance();
		currentResults = new SearchResult[0];
		data.receiveSearchMessages(this);
		data.applyAsProgressSignalReceiver(this);
		
		this.setTitle("Tazup client");
		this.setSize(500, 330);
		this.setLocationRelativeTo(null);
		this.setDefaultCloseOperation(EXIT_ON_CLOSE);
		this.setIconImage(IconDatabase.getInstance().getIcon("tazup"));
		
		this.setJMenuBar(
			MenuManager.getInstance().createJMenuBar()	
		);
		
		final JFrame me = this;
		ActionDatabase adb = ActionDatabase.getInstance();
		adb.addAction("exit", new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				me.dispose();
			}
		});
		
		JPanel panel = new JPanel();
		//! this.getContentPane() doesn`t exists
		this.add(panel);
		panel.setLayout(null);
		panel.setToolTipText("Tazup is the future of file sharing");
		
		JButton quitButton = new JButton("QUIT");
		quitButton.setBounds(390, 10, 100, 20);
		quitButton.addActionListener(ActionDatabase.getInstance().getAction("exit"));
		quitButton.setToolTipText("Button component");
		panel.add(quitButton);
		
		progressBar = new JProgressBar(0, 1000);
		progressBar.setValue(0);
		progressBar.setStringPainted(true);
		progressBar.setBounds(10, 250, 480, 20);
		progressBar.setString("Download progress");
		panel.add(progressBar);
		
		final JTextField searchField = new JTextField();
		searchField.setBounds(140, 10, 100, 20);
		searchField.setToolTipText("Type the search keywords here.");
		panel.add(searchField);
		
		resultsTable = new JTable();
		resultsTable.setToolTipText("Search results");
		resultsTable.setPreferredScrollableViewportSize(new Dimension(480, 140));
		resultsTable.setFillsViewportHeight(true);
		resultsTable.getTableHeader().setReorderingAllowed(false);
		receiveSearchMessage();
		
		JScrollPane resultsPane = new JScrollPane(resultsTable);
		resultsPane.setBounds(10, 70, 480, 140);
		panel.add(resultsPane);
		
		JLabel searchLabel = new JLabel("Search keyword:");
		searchLabel.setLabelFor(searchField);
		searchLabel.setBounds(10, 10, 140, 20);
		panel.add(searchLabel);
		
		adb.addAction("search", new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				SearchQuery sq = SearchQuery.parseKeywords(searchField.getText());
				data.getSearchInstance().search(sq);
			}
		});
		
		JButton searchButton = new JButton("Search");
		searchButton.setBounds(250, 10, 100, 20);
		searchButton.addActionListener(ActionDatabase.getInstance().getAction("search"));
		searchButton.setToolTipText("Search for this keyword");
		panel.add(searchButton);
		
		JLabel resultsLabel = new JLabel("Search results are to be listed here:");
		resultsLabel.setLabelFor(resultsTable);
		resultsLabel.setBounds(10, 40, 340, 20);
		panel.add(resultsLabel);
		
		adb.addAction("download", new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				progressBar.setValue(0);
				for (int i = 0; i < currentResults.length; i++)
					if (resultsTable.getSelectionModel().isSelectedIndex(i)) {
						if (data.getDownloadInstance().startDownloadFile(currentResults[i])) {
						
						} else {
							JOptionPane.showMessageDialog(null,
							                              "Unable to start the download.");
						}
					}
			}
		});
		
		JButton downloadButton = new JButton("Download");
		downloadButton.setBounds(340, 220, 150, 20);
		downloadButton.setToolTipText("Download the selected file from the list.");
		downloadButton.addActionListener(ActionDatabase.getInstance().getAction("download"));
		panel.add(downloadButton);
		
		JLabel downloadLabel = new JLabel("Select a file from the list and click download.");
		downloadLabel.setBounds(10, 220, 330, 20);
		downloadLabel.setLabelFor(downloadButton);
		panel.add(downloadLabel);
	}
	
}
