package gui;

import java.awt.Image;
import java.awt.Point;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import java.util.HashMap;

import javax.imageio.ImageIO;

//singleton
public class IconDatabase {
	//http://media.smashingmagazine.com/wp-content/uploads/images/free-icons-round-up/free-icons-round-up-41.png
	protected static final String palette_filepath = "data/gui/icons.png";
	protected static final int palette_offset_x = 20; 
	protected static final int palette_offset_y = 28;
	protected static final int icon_width = 20;
	protected static final int icon_height = 20;
	protected BufferedImage palette;
	protected HashMap<String, Point> icon_position; 
	protected HashMap<String, Image> icon_cache;
	
	protected IconDatabase(){
		try {
			this.palette = ImageIO.read(new File(palette_filepath));
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		this.icon_position = new HashMap<String, Point>();
		this.icon_cache = new HashMap<String, Image>();
		
		// TODO Loading icon information from file
		this.icon_position.put("tazup", new Point(21,10));
		this.icon_position.put("exit", new Point(5,3));
		this.icon_position.put("open", new Point(2,2));
		this.icon_position.put("a", new Point(8,15));
		this.icon_position.put("b", new Point(13,11));
		this.icon_position.put("c", new Point(18,4));
		this.icon_position.put("d", new Point(1,18));
		this.icon_position.put("e", new Point(0,19));
		this.icon_position.put("aa", new Point(9,15));
		this.icon_position.put("ab", new Point(14,11));
		this.icon_position.put("ac", new Point(19,4));
		this.icon_position.put("ad", new Point(2,18));
		this.icon_position.put("ae", new Point(1,19));
	}
	
	public static IconDatabase getInstance(){
		if(_instance == null){
			_instance = new IconDatabase();
		}
		return _instance;
	}
	
	public Image getIcon(String name){
		if(!this.icon_position.containsKey(name)){
			//throw exception
			return null;
		}
		
		if(this.icon_cache.containsKey(name)){
			return this.icon_cache.get(name);
		}
		
		Point p = this.icon_position.get(name);
		Image result = this.palette.getSubimage(
					palette_offset_x * p.x,
					palette_offset_y * p.y,
					icon_width,
					icon_height
				);
		this.icon_cache.put(name, result);
		
		return result;
	}
	
	private static IconDatabase _instance = null; 
}
