package gui;

import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;
import java.util.HashMap;

import javax.swing.ImageIcon;
import javax.swing.JCheckBoxMenuItem;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.KeyStroke;

//Singleton class

public class MenuManager {
	protected static MenuManager _instance = null;
	HashMap<Integer, MenuItemDescriptor> data;
	
	//Load ini data
	protected MenuManager(){
		//TODO Load from ini file
		this.data = new HashMap<Integer, MenuItemDescriptor>();
		this.loadMenuDescriptors();
	}
	
	//There is a copy constructor, so in the initialization we can use the same
	//MenuItemDescriptor reference.
	protected void add(MenuItemDescriptor d){
		this.data.put(d.id, new MenuItemDescriptor(d));
	}
	protected void add(MenuItemDescriptor d, int parent_id){
		if(!this.data.containsKey(parent_id)){
			//!throw -- just because of bad initialization
			this.add(d);
			return;
		}
		MenuItemDescriptor cache = d.parent;
		d.parent = this.data.get(parent_id);
		
		this.add(d);
		
		d.parent = cache;
	}
	
	public JMenuBar createJMenuBar(){
		JMenuBar menubar = new JMenuBar();
	
		for(MenuItemDescriptor d : this.data.values()){
			this.createMenuComponent(menubar, d);
		}
		
		return menubar;
	}
	
	protected boolean hasChild(MenuItemDescriptor d){
		for(MenuItemDescriptor iter : this.data.values()){
			if(d == iter.parent) return true;
		}
		return false;
	}
	
	protected void createMenuComponent(JMenuBar menubar, MenuItemDescriptor d){
		
		//This component was already created
		if(d.component != null){
			return;
		}
		
		//Before child is created, parent must be also be created. 
		if(d.parent != null && d.parent.component == null){
			this.createMenuComponent(menubar, d.parent);
		}
		
		//D is a parent component
		if(this.hasChild(d)){
			JMenu menu = new JMenu(d.name_to_show);
			d.component = menu;
		}
		//D is a list component
		else{
			JMenuItem item;
			if(d.is_checkbox) 
				item = new JCheckBoxMenuItem(d.name_to_show, new ImageIcon(IconDatabase.getInstance().getIcon(d.icon_name)));
			else
				item = new JMenuItem(d.name_to_show, new ImageIcon(IconDatabase.getInstance().getIcon(d.icon_name)));
			d.component = item;
		}
		
		//Root component
		if(d.parent == null){
			menubar.add(d.component);
		}
		//Child component
		else{
			if(d.separator_before_me && d.parent != null && d.parent.component instanceof JMenu){
				((JMenu)d.parent.component).addSeparator();
			}
			
			d.parent.component.add(d.component);
		}
		
		if(d.hotkey >= 0) d.component.setMnemonic(d.hotkey);
		if(d.tooltip != null) d.component.setToolTipText(d.tooltip);
		if(d.action != null) d.component.addActionListener(d.action);
		if(d.action != null && d.shortcut != null) d.component.setAccelerator(d.shortcut);
	}
	
	protected void loadMenuDescriptors(){
		MenuItemDescriptor d = new MenuItemDescriptor();
		d.id=0;
		d.parent=null;
		d.action=null;
		d.icon_name=null;
		d.name_to_show="File";
		d.hotkey=KeyEvent.VK_F;
		d.tooltip="File options";
		this.add(d);
		
		d.id=1;
		d.name_to_show="View";
		d.hotkey=KeyEvent.VK_V;
		d.tooltip="View options";
		this.add(d);
		
		d.id=2;
		d.name_to_show="Search";
		d.hotkey=KeyEvent.VK_S;
		d.tooltip="Search options";
		this.add(d);
		
		d.id=3;
		d.name_to_show="Help";
		d.hotkey=KeyEvent.VK_H;
		d.tooltip="Find some external help";
		this.add(d);
		
		d.id=5;
		d.name_to_show="SearchBar";
		d.icon_name="a";
		d.tooltip=null;
		d.is_checkbox=true;
		this.add(d, 1);
		d.is_checkbox=false;
		
		d.id=6;
		d.name_to_show="PeerList";
		d.icon_name="b";
		d.hotkey=KeyEvent.VK_P;
		d.is_checkbox=true;
		this.add(d, 1);
		d.is_checkbox=false;
		
		d.id=7;
		d.name_to_show="Statistics Center";
		d.icon_name="c";
		d.hotkey=KeyEvent.VK_T;
		d.is_checkbox=true;
		this.add(d, 1);
		d.is_checkbox=false;
		
		d.id=8;
		d.name_to_show="Online help";
		d.icon_name="d";
		d.hotkey=KeyEvent.VK_O;
		this.add(d, 3);
		
		d.id=9;
		d.name_to_show="Search for peers";
		d.icon_name="e";
		d.hotkey=KeyEvent.VK_P;
		this.add(d, 2);

		d.id=10;
		d.name_to_show="Share";
		d.icon_name="aa";
		d.tooltip="Share some resource";
		d.hotkey=KeyEvent.VK_S;
		this.add(d, 0);

		d.id=11;
		d.name_to_show="File";
		d.icon_name="ab";
		d.tooltip="Share one file";
		d.hotkey=KeyEvent.VK_F;
		this.add(d, 10);

		d.id=12;
		d.name_to_show="Files";
		d.icon_name="ac";
		d.tooltip="Share multiple files";
		d.hotkey=KeyEvent.VK_G;
		this.add(d, 10);

		d.id=13;
		d.name_to_show="Directory";
		d.icon_name="ad";
		d.tooltip="Share one directory";
		d.hotkey=KeyEvent.VK_D;
		d.separator_before_me = true;
		this.add(d, 10);
		d.separator_before_me = false;
		
		d.id=14;
		d.name_to_show="Directories";
		d.icon_name="ae";
		d.tooltip="Share multiple directories";
		d.hotkey=KeyEvent.VK_S;
		this.add(d, 10);
		

		d.id=15;
		d.name_to_show="Exit";
		d.icon_name="exit";
		d.tooltip="Exit application";
		d.hotkey=KeyEvent.VK_W;
		d.shortcut=KeyStroke.getKeyStroke(d.hotkey, ActionEvent.CTRL_MASK);
		d.action= ActionDatabase.getInstance().getAction("exit");
		d.separator_before_me = true;
		this.add(d, 0);
		d.separator_before_me = false;
		d.action=null;
		
	}
	
	public static MenuManager getInstance(){
		if(_instance == null){
			_instance = new MenuManager();
		}
		return _instance;
	}
}
