package modules;

import java.io.IOException;
import java.util.concurrent.TimeUnit;

import types.*;

/*
 * handles all communication
 * 
 * advantages of this centralized aproach:
 * 		enables testing by simulating network
 * 		enables stats
 * 
 * sitting on port, reading packets
 * packet comes, NC sends message to responsible module (returns from recieveMessage)
 * 		+ sends message to PeerManagementModule about active peer
 */

public interface Communication extends Module {
	
	void sendMessage(Message msg) throws IOException;
	
	// called by modul
	// should be sort of blocking - modul sleeps until there is message for it
	// throws Interrupted exception, if thread was interrupted while waiting for message
	// usualy u wanna attempt to read message again
	// btw. module, which in run method in cycle uses this method, in exit method should also interrupt its thread
	Message receiveMessage(ModulType modul) throws InterruptedException;
	// if no message is available in specified timeout, returns null
	Message receiveMessage(ModulType modul, long timeout, TimeUnit unit) throws InterruptedException;
}
