package modules;

import java.io.IOException;
import java.util.HashMap;
import java.util.Random;

import auxiliary.Convert;
import auxiliary.SHA1;

import types.*;

//!TODO synchronized
//!TODO 
/*
 * Handles download threads
 * 
 * holds reference to NetworkCommunication and DataStorage
 * 
 * when user initiates download, DM creates DownloadThread
 * DM is listening to NC messages and forwarding them to responsible DT
 * DM is listening to DT messages and forwarding them to NC
 * 		there are possible messages for writing data to DS
 * (DT <-> DM <-> NC; DT -> DM -> DS)
 */

/* File download protocol:
 * 
 * */

public class DownloadModul implements Module {
	
	//!TODO in developer version it is relative, in production absolute
	public static String download_folder = "share";
	
	//!TODO REFACTOR: redundancy of declarations through modules
    private Communication comm;
    //!TODO REFACTOR: DataStorage is now singleton.  
    private DataStorage data;
    
    //For each download is a thread initialized. (SequenceID is agreed in the first uploader/downloader communication)
    //DownloadModul distributes messages to its sub threads (identified by SequenceID)
    private HashMap<SequenceID, DownloadThread> threads;
    
    //!TODO
    private HashMap<StreamID, SequenceID> stream_to_seq; 
    private boolean running = true;
    private Thread myThread;
	
	public DownloadModul() {
		this.data = DataStorage.getInstance();
		this.comm = data.getCommunicationInstance(); 
		this.threads = new HashMap<SequenceID, DownloadThread>();
	}
	
	//!TODO - remove (all similar)s
	public PeerID getSourceId(){
		return this.data.myPeerID();
	}
	public PeerInfo getPeerInfoFromPeerID(PeerID id){
		return this.data.findPeerByID(id);
	}
	
	//!TODO - Threads management
	//Pause / Continue / Stop downloading
	//!TODO
	//Active transfers are stored in DataStorage.DownloadDecsriptors()
	
	private DownloadedFileDescriptor.Uploader getUploaderFromSearchResult(DownloadedFileDescriptor dd, SearchResult sr){
		Random rand = new Random();
		
		DownloadedFileDescriptor.Uploader ud = dd.new Uploader(); 
		ud.peer_id = sr.peer();
		ud.sequence_id = new SequenceID(rand.nextInt());
		//All is agreed cross TCP. 
		ud.stream_id = data.generateNewStreamID();
		return ud;
	}
	
	//Called by some application controller (Client).
	//If the file is already downloaded some thread, at this is some new peer, then it will be downloaded
	//--in a new thread (see this.startDownloadFileInNewThread()) 
	public boolean startDownloadFile(SearchResult sr) {
		//Check if download of that file is already in progress
		DownloadedFilesDatabase DFD = DataStorage.getInstance().DownloadedFiles();
		DownloadedFileDescriptor searched_dfd = (DownloadedFileDescriptor) DFD.getDescriptorByChecksum(sr.checksum());

		DownloadedFileDescriptor dd = null;
		
		//Application is already downloading this file from someone.  
		if(null != searched_dfd && !searched_dfd.uploaders.isEmpty()){
			//I already download this file from that peer (so nothing to do)
			if(null != searched_dfd.getUploaderByPeerIdValue(sr.peer().value())){
				DataStorage.getInstance().debugMessage("Download of file " + sr.filename() + " from peer " + sr.peer().value() + " already in progress.");
				return false;
			}
			//DownloadDescriptor will be searched once more in DownloadDatabase. 
			DataStorage.getInstance().debugMessage("Download of file " + sr.filename() + " is in progress from other peer. Now we try to launch parallel download in other DownloadThread.");
			return this.startDownloadInNewThread(sr);
		}
		//Descriptor exists, but nobody is uploading (probably just loaded from database at init)
		else if(null != searched_dfd){
			dd = searched_dfd;
		}
		//It is necessary to start pure new download. 
		else{
			dd =  new DownloadedFileDescriptor(
				sr.filename(), 
				DownloadModul.download_folder, 
				sr.checksum()
			);
		}
		
		return this.startDownloadThread(dd, this.getUploaderFromSearchResult(dd, sr));
	}
	//
	private boolean startDownloadInNewThread(SearchResult sr){
		DownloadedFilesDatabase DFD = DataStorage.getInstance().DownloadedFiles();
		if(null == DFD.getFilenameByChecksum(sr.checksum())){
			DataStorage.getInstance().debugMessage("Download of file " + sr.filename() + " is not in progress (nothing happened).");
			return false;
		}
		DownloadedFileDescriptor dfd = (DownloadedFileDescriptor) DFD.getDescriptorByChecksum(sr.checksum());

		return this.startDownloadThread(dfd, this.getUploaderFromSearchResult(dfd, sr));
	}
	
	//--Maybe some eased call (relatively to DownloadDatabase). 
	public boolean startDownloadFile(DownloadedFileDescriptor dd, DownloadedFileDescriptor.Uploader ud){
		return this.startDownloadThread(dd, ud);
	}
	
	//Adds a peer from which I download this file
	//!TODO check if this family of functions is ok
	private boolean startDownloadThread(DownloadedFileDescriptor dd, DownloadedFileDescriptor.Uploader ud){
		if(DataStorage.getInstance().SharedFiles().containsChecksum(dd.getTargetChecksum())){
			DataStorage.getInstance().debugMessage("It is impossible to download same file (indetified by checksum) as one of the shared files. File=" + dd.getFilepath());
			return false;
		}
		
		//Store this uploader in DownloadedFileDescriptor
		dd.uploaders.add(ud);
		
		//Creates DownloadThread with DownloadedFileDescriptor (which will be also stored in DataStorage)
		DownloadThread result = new DownloadThread(this, dd, ud);
		data.DownloadedFiles().add(dd);
		this.threads.put(ud.sequence_id, result);
		
		//!TOD remove
		//data.debugMessage("Thread put SeqID: " + ud.sequence_id);
		
		(new Thread(result)).start();
		
		return true;
	}
	
	//Called by DownloadThread (used in ID agreement -- when SequenceIDs collide)
	//--generates new SequenceID (which is not used) end returns it
	//--also changes the this.threads database
	public SequenceID changeSequenceId(SequenceID thread_id){
		if(this.threads.containsKey(thread_id)){
			DataStorage.getInstance().debugMessage("Tried to change thread_id which not exists in DownloadModul.");
			return thread_id;
		}
		
		Random rand = new Random();
		SequenceID new_id;
		do{
			new_id = new SequenceID(rand.nextInt());
		}while(this.threads.containsKey(new_id));
		
		this.threads.put(new_id, this.threads.get(thread_id));
		this.threads.remove(thread_id);
		
		return new_id;
	}
	
	//Same as in UploadModul
	//In each case, that DownloadThread will be killed
	//But first it tries to successfully finish download (in DownloadThread.run() method)
	//--if not successful, it will be killed. 
	public void finishDownload(SequenceID id){
		if(id == null)
			return;
		if(!this.threads.containsKey(id))
			return;
		
		//All what is necessary in destruction will be handled in that concrete thread
		this.threads.get(id).exit();
		this.threads.remove(id);
	}
	
	//Calls finish download for all threads which download that file
	public void finishDownload(DownloadedFileDescriptor dfd){
		for(DownloadedFileDescriptor.Uploader up : dfd.uploaders){
			this.finishDownload(up.sequence_id);
		}
	}
	
	public static void messageTypeForUpload(Message message){
		message.messageType |= 0x10; 
	}
	public static void messageTypeForDownload(Message message){
		message.messageType &= 0x0f; 
	}
	public static void messageTypeForParse(Message message){
		message.messageType &= 0x0f; 
	}
	
	// called by DownloadThread -- sends to UploadModule
	public void sendMessage(Message message) {
		if(!message.streamClosed){
			message.encode();
			message.dataLength = ((message.data == null) ? 0 : message.data.length);
			DownloadModul.messageTypeForUpload(message);
		}
		try {
			comm.sendMessage((Message)message);
		} catch (IOException e) {
			data.raiseError("Unable to send message");
		}
	}
	
	private void serializeMessage(Message message){
		//DataStorage.getInstance().debugMessage("Download modul serialize message. SequenceID: " + message.identifier + " type: 0x" + Convert.byteToAlphaHex(message.messageType));
		//System.out.println("Download modul serialize message: \n" + message.toString());

		if(message.streamClosed){
			//!TODO stream_id -> sequence_id
			//thread.cancelResponse(message);
			//Notify communication, that the conversation was canceled. 
			this.sendMessage(message);
			return;
		}
		
		if(!this.threads.containsKey(message.identifier)){
			//DataStorage.getInstance().debugMessage("Unhandled message in Download module: \n" + message.toString());
			return;
		}
		DownloadThread thread = threads.get(message.identifier);
		
		DownloadModul.messageTypeForParse(message);
		DownloadMessage.parse(message);
		
		if(message.representation instanceof DownloadMessage.InitMessage){
			thread.initResponse(message);
			return;
		}
		if(message.representation instanceof DownloadMessage.PartMessage){
			thread.addPartToRespond(message);
			return;
		}
		if(message.representation instanceof DownloadMessage.CancelMessage){
			thread.cancelResponse(message);
			return;
		}
		
		DataStorage.getInstance().debugMessage("Download modul haven't found MessageDataRepresentation ("+((message.representation==null)?"null":"not null")+") for message: \n" + message.toString());
	}
	
    @Override
    public void run() {
    		myThread = Thread.currentThread();
            while (running) {
            	Message message = null;
                try {
                    message = comm.receiveMessage(ModulType.DOWNLOAD);
                } catch (InterruptedException e) {
                    //!TODO
                    //throw new NotImplementedException();
                	data.debugMessage("UM interrupted");
                	continue;
                }
                //No new variables, just functionality is added. 
				this.serializeMessage(message);
            }
    }

    @Override
    public void exit() {
    	//!TODO remove
    	data.debugMessage("DM exit()");
        running = false;
            
        //Stop all threads running
        for(DownloadThread dt : this.threads.values()){
            dt.exit();
        }
            
        if (myThread != null) myThread.interrupt();
    }
    
    //for stats
    public int activeThreads() {
    	return threads.size();
    }
	
}
