package modules;

//!TODO synchronized
//!TODO -- check if all file serialization is called from the DownloadThread.run() method 
//--(it is the slowest part and the most errors occur there)
//!TODO timeout for descriptor backup  ( also time elapsed )


/*
 * Downloads a file from a Peer. 
 * --receives messages from DownloadModul 
 * --send messages through DownloadModul 
 * 
 * The progress is stored in DownloadedFileDescriptor (also accessible in DataStorage.DownloadedFileDescriptors)
 * 
 * First, the download is initialized (if the peer is online, has the files, wants to send it, uses that sequence id)
 * 
 */
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.Random;
import java.util.concurrent.ArrayBlockingQueue;

import javax.xml.crypto.Data;

import auxiliary.Convert;
import auxiliary.SHA1;

import types.DownloadedFileDescriptor;
import types.Message;
import types.PeerInfo;
import types.SequenceID;
import types.SharedFileDescriptor;

public class DownloadThread implements Runnable {
	//!TODO -- change back
	static public final int max_requested_not_received_parts = 20;
	//After downloaded 
	static public final int backup_each_parts = 100;
	
	//In the Descriptor is stored all what is needed for a successful download. 
	private DownloadedFileDescriptor dd; 
	private DownloadedFileDescriptor.Uploader ud; 
	private DownloadModul parent; 
	private boolean started = false;
	private boolean stoped = false;
	
	private ArrayBlockingQueue<Message> partsToReceive = new ArrayBlockingQueue<Message>(max_requested_not_received_parts); 
	private int requested_minus_received = 0;
	
	public SequenceID getSequenceId(){
		if(ud != null){
			return ud.sequence_id;
		}
		return null;
	}
	
	//Events?
	public DownloadThread(DownloadModul parent, DownloadedFileDescriptor dd, DownloadedFileDescriptor.Uploader ud) {
		this.dd = dd;
		this.ud = ud;
		DataStorage.getInstance().debugMessage("Created thread SeqID: " + ud.sequence_id);
		this.parent = parent;
		this.started = false;
		this.stoped = false;
	}
	
	//!TODO -- check if this procedure exited correctly
	private boolean setMessageFromUploaderDescriptor(Message result)
	{
		result.identifier = this.ud.sequence_id;
		result.sourceID = this.parent.getSourceId();
		result.streamID = this.ud.stream_id;
		result.targetID = this.ud.peer_id;
		result.remotePeer = this.parent.getPeerInfoFromPeerID(this.ud.peer_id);
		
		//The recipient was not found. 
		if(result.remotePeer == null){
			DataStorage.getInstance().debugMessage("remotePeer for " + this.ud.peer_id + " is null.\nActive peers: ");
			return false;
		}
		if (result.remotePeer.id() == null) {
			DataStorage.getInstance().debugMessage("remote peer is inactive");
			return false;
		}
		return true;
	}
	
	//Sends message (request) to UploadThread for start download
	public void initDownload(){
		DataStorage.getInstance().debugMessage("initDownload");
		
		DownloadMessage init = new DownloadMessage();
		
		if (!setMessageFromUploaderDescriptor((Message)init)) {
			// TODO by Niemand... what to do if Message creation not succesful;
			return;
		}
		
		DownloadMessage.InitMessage data = init.new InitMessage();
		init.representation = data;
		data.init_sha = dd.getChecksum();
		data.init_type = DownloadMessage.InitType.REQUEST;
		
		this.parent.sendMessage(init);
	}
	public void initResponse(Message received){
		DataStorage.getInstance().debugMessage("initResponse");
		
		DownloadMessage.InitMessage data = (DownloadMessage.InitMessage) received.representation;
		if(data.init_type == DownloadMessage.InitType.OK){
			
			//We don't want to download/request for parts if not allocated. 
			synchronized(this) {
				if(this.started)
					return;
				
				DataStorage.getInstance().debugMessage("Response ok for file: "+dd.getFilepath()+" fileSize="+data.file_size);
				
				if(!this.dd.allocate(data.file_size)){
					//!TODO -- error in downloading. 
					DataStorage.getInstance().debugMessage("File: " + dd.getFilepath() + " not allocated." );
					return;
				}
				this.started = true;
				DataStorage.getInstance().debugMessage("Thread.started = true");
			}
			
		}
		if(data.init_type == DownloadMessage.InitType.REGEN){
			this.ud.sequence_id = this.parent.changeSequenceId(this.ud.sequence_id);
			this.initDownload();
		}
		if(data.init_type == DownloadMessage.InitType.CANCEL){
			this.exit();
		}
	}
	
	public void addPartToRespond(Message message){
		//!TODO part to respond: -1 ?
		//!TODO remove
		DataStorage.getInstance().debugMessage("DT add message of part to respond: " + Convert.bytesToInt(message.data,0));
		this.partsToReceive.add(message);
	}
	
	//It occur that the last parts will be sent more times. 
	private void partDownload() throws IOException{
		synchronized(this){
			if(this.finished())
				return;
			
			this.partDownload(this.dd.getMissingPart());
		}
	}
	private void partDownload(int part_id){
		if(part_id < 0 || part_id >= this.dd.getPartsCount()){
			DataStorage.getInstance().debugMessage("Requested non-existing part " + part_id + " in " + this.dd.getFilepath());
			return;
		}
		DownloadMessage part = new DownloadMessage();
		setMessageFromUploaderDescriptor(part);
		
		DownloadMessage.PartMessage data = part.new PartMessage();
		part.representation = data;
		data.part_id = part_id;
		data.part_data = null;
		DataStorage.getInstance().debugMessage("partDownload: " + data.part_id);
		
		this.parent.sendMessage(part);
	}
	private void partResponse(Message received){
		DownloadMessage.PartMessage data = (DownloadMessage.PartMessage) received.representation;
		boolean error = false;
		
		if(!this.dd.addPart(data.part_id, data.part_data))
			error = true;
		
		if(error){
			DataStorage.getInstance().debugMessage(
					"Part " + data.part_id + " of file " + dd.getFilename() + " was not serialized. " +
							"It was requested once more. Message: \n" + received.toString());
			//!TODO -- uncommment
			//this.partDownload(data.part_id);
		}
		else{
			DataStorage.getInstance().debugMessage("partResponse OK: " + data.part_id);
		}
	}
	public void cancelResponse(Message received){
		//Stream closed
		if(!received.streamClosed){
			if(!(received.representation instanceof DownloadMessage.CancelMessage)){
				DataStorage.getInstance().debugMessage("DownloadThread.cancelResponse message.representation is not instanceof CancelMessage. Message ignored: " + received.toString());
				return;
			}
		}
		
		String why = "";
		if(received.streamClosed){
			why = "stream closed message received";
		}
		else{
			why = ((DownloadMessage.CancelMessage) received.representation).cancelReason();
		}
		DataStorage.getInstance().debugMessage("DownloadThread SeqID: " + this.getSequenceId() + " because "+why+".");
		this.exit();
	}
	
	public void exit(){
		this.started = true;
		this.stoped = true;
	}
	
	//If all was downloaded
	public boolean finished(){
		return dd.finished() || this.stoped;
	}

	//In each case, this DownloadThread will be killed
	//But first it tries to successfully finish download (in DownloadThread.run() method)
	//--if not successful, it will be killed. 
	private boolean finish(){
		String error_message = null;
		
		synchronized(this) {
			if(!this.dd.uploaders.contains(this.ud)){
				//Download already finished
				this.stoped = true;
				return true;
			}
			
			//remove from DownloadModul.threads
			this.parent.finishDownload(this.getSequenceId());
			//Just remove information about that uploader. 
			this.dd.uploaders.remove(this.ud);
			
			//Creates a CancelMessage for the Uploader. 
			DownloadMessage dm = new DownloadMessage();
			this.setMessageFromUploaderDescriptor(dm);
			
			DownloadMessage.CancelMessage cancel = dm.new CancelMessage();
			cancel.cancel_type = DownloadMessage.CancelType.NOT_SPECIFIED;
			dm.representation = cancel;
	
			//In each case it tries to completize the file. 
			//--if finished, store final file, remove parts file... 
			if(!dd.completize()){
				error_message = "Download file cannot be completized";
			}
			
			//If completized, do the other finalize things. 
			//--Check if checksum is ok, if not, launch once more...
			if(error_message == null){
					//Downloaded file is now shared file. 
					DataStorage.getInstance().SharedFiles().add((SharedFileDescriptor)dd);
					DataStorage.getInstance().DownloadedFiles().remove(dd);
					
					DataStorage.getInstance().debugMessage("DownloadThread.finishDownload filepath=" + dd.getFilepath() + " successful");
					cancel.cancel_type = DownloadMessage.CancelType.FINISHED;
			}
			else{
				//Backup all what is possible. 
				//--(It is even better, that this method is called privately only from run() method, all the file transfer should be done from there)
				this.dd.backup();
				DataStorage.getInstance().debugMessage("DownloadModul.finishDownload filepath=" + dd.getFilepath() + " NOT successful because " + error_message);
			}
			//In each case if DownloadThread has been finished the corresponding UploadThread must be finished to. 
			//Send to upload thread, that this communication is canceled. 
			//--no SequenceID collision occurs, because it will be new on the both sides. 
			this.parent.sendMessage(dm);
		}
		return (error_message == null);
	}
	
	@Override
	public void run() {
		//!TODO -- test new implementation
		initDownload();
		
		//No need for synchronized this.started .. it is an infinite loop 
		while(!this.started){
			try {
				Thread.sleep(100);
			} catch (InterruptedException e) {
				continue;
			}
			//DataStorage.getInstance().debugMessage("DownloadThread " + this.getSequenceId() + " not started.");
		}
		
		DataStorage.getInstance().debugMessage("DownloadThread " + this.getSequenceId() + "started.");
		
		while(!this.finished()){
			DataStorage.getInstance().signalDownloadProgress();
			
			/*//Smaller network traffic. 
			if(this.requested_minus_received==0 || (
					this.requested_minus_received < max_requested_not_received_parts 
					&& !this.partsToReceive.isEmpty())
			 * */
			if(this.requested_minus_received < max_requested_not_received_parts ){
				try {
					this.partDownload();
				} catch (IOException e) {
					//Nothing to do, the part will be requested once more (it is assured with getMissingPart functionality)
					continue;
				}
				this.requested_minus_received++;
			}
			if(!this.partsToReceive.isEmpty()){
				try {
					this.partResponse(this.partsToReceive.take());
				} catch (InterruptedException e) {
					//Nothing to do, if interrupted while waited, then I will try it again. 
					continue;
				}
				this.requested_minus_received--;
			}
			if(this.dd.getDownloadedPartsCount() % backup_each_parts == (backup_each_parts - 1)){
				this.dd.backup();
			}
			
			/*
			//!TODO -- remove slow down sleep
			try {
				Thread.sleep(50);
			} catch (InterruptedException e) {
				continue;
			}*/
		}
		//Error/Debug message handling is done in that function. 
		this.finish();
		
		DataStorage.getInstance().signalDownloadProgress();
		DataStorage.getInstance().debugMessage("DownloadThread " + this.getSequenceId() + " finished.");
	}
	
	@Override
	public void finalize(){
		if(this.dd != null && !this.dd.finished()){
			this.dd.backup();
		}
	}

}
