package modules;

import java.util.HashSet;

import auxiliary.SHA1;
import types.DownloadedFileDescriptor;
import types.SharedFileDescriptor;

/* Keeps track of files which are currently downloaded and reference to threads, which 
 * 
 * */

//Many functions are the same as in SharedFilesDatabase
public class DownloadedFilesDatabase extends SharedFilesDatabase {

	private static final long serialVersionUID = 4576631124415549982L;
	
	//Load from ini file
	public DownloadedFilesDatabase(){
		super();
	}
	@Override
	public String configPath(){
		return DataStorage.getInstance().configPath() + DataStorage.down_db_init_file;
	}
	
	@Override
	public boolean add(SharedFileDescriptor sfd){
		//I don`t care if the file exists (it will not, if download is not finished) 
		//I just want to add this Descriptor into database;
		if(this.data.add(sfd)){
			this.indexByChecksum.put(sfd.getTargetChecksum(), sfd);
			return true;
		}
		return false;
	}

	//Called from ini file
	@Override
	protected boolean addDescriptor(String filename, String folder, SHA1 hash){
		return this.add(new DownloadedFileDescriptor(filename, folder, hash));
	}
	
	@Override
	//!TODO -- load also peers which uploaded
	//!TODO -- automatically start downloading from them (if peer exists in the peer list -- new StreamIDs should be generated and so on) 
	public void init(){
		synchronized(this) {
			this.loadFromFile(this.configPath());
			for(SharedFileDescriptor sfd : this.data){
				DownloadedFileDescriptor dfd = (DownloadedFileDescriptor) sfd; 
				//Loads the parts file and opens the buffer file for download. 
				dfd.load();
			}
		}
	}
	@Override
	//!TODO -- see TODO at this.init();
	public boolean backup(){
		synchronized(this){
			for(SharedFileDescriptor sfd : this.data){
				DownloadedFileDescriptor dfd = (DownloadedFileDescriptor) sfd; 
				dfd.backup();
			}
			return this.backupToFile(this.configPath());
		}
	}

	public double getTotalProgress() {
		double ret = 0;
		for (SharedFileDescriptor sfd: this.data)
			if (((DownloadedFileDescriptor)sfd).finished())
				ret += 1;
			else
				ret += ((DownloadedFileDescriptor)sfd).getProgress();

		return ret / this.data.size();
	}
}
