package modules;

import java.util.HashSet;

/** IDGenerator generates a non-random but unique integers. **/
/** Unneeded integers can be returned to be reused again.   **/

/** NOTE: IDGenerator is quite foolproof. Returning the same value **/
/** twice in a row (without reobtaining it in the meantime) or     **/
/** returning a value that has not even been really generated      **/
/** should not invalidate its functionality.                       **/

public class IDGenerator {
        /* private members */
        private int current;
        private HashSet<Integer> returned;
        private static final int max_reuse = 150;
        
        /* constructor */
        public IDGenerator() {
            current = 0;
            returned = new HashSet<Integer>();
        }
        
        /* return a previously used ID */
        public synchronized void returnUsedID(int id) {
            if (id < current && returned.size() < max_reuse)
                returned.add(new Integer(id));
        }
        
        /* generate new ID */
        public synchronized int generateNewID() {
            if (returned.isEmpty())
                return current++;
            
            Integer ret = returned.iterator().next();
            returned.remove(ret);
            return ret.intValue();
        }
        
}
