package modules;

import java.util.HashMap;
import java.util.concurrent.ArrayBlockingQueue;

import types.Message;
import types.ModulType;

public abstract class NetworkListener implements Module {
	
	protected DataStorage dataStorage;
	protected HashMap<ModulType, ArrayBlockingQueue<Message> > msgStorage;
	
	protected void processMessage(Message msg) {
		if (msg == null)
			throw new NullPointerException();
		
		boolean closedMessage = msg.streamClosed && msg.streamID != null;
		boolean forPeerMngOnly = ModulType.PEER_MANAGEMENT.equals(msg.targetModule());
		boolean targetIDIsMine = dataStorage.myPeerID().equals(msg.targetID) || closedMessage;
		
		boolean stored = false;
		
		if (targetIDIsMine) {
			storeMessage(msg, msg.targetModule());
			stored = true;
		}
		
		if (! ((forPeerMngOnly && targetIDIsMine) || closedMessage)) {
			Message msgh = Message.messageHeaderFromMessage(msg);
			if (msgh.messageType == 0x02)	// if search query also data with initiator info needed
				msgh.data = msg.data.clone();
			storeMessage(msgh, ModulType.PEER_MANAGEMENT);	
			stored = true;
		}
		
		if (!stored)
			dataStorage.debugMessage("impossible !");
	}
	
	private void storeMessage(Message msg, ModulType targetModule) {
		if (msg.streamID != null){
			//dataStorage.debugMessage("trying to store message");
		}
		boolean stored = false;
		while (!stored) {
			try {
				msgStorage.get(targetModule).put(msg);
				if (msg.streamID != null){
					//dataStorage.debugMessage("message stored");
				}
				stored = true;
			} catch (InterruptedException e) {
				stored = false;
	 		}
		}
	}
}
