package modules;

import java.lang.InterruptedException;
import java.net.InetAddress;
import auxiliary.Convert;
import java.net.UnknownHostException;
import java.util.Timer;
import java.util.TimerTask;
import java.io.IOException;
import java.util.TreeMap;
import java.util.SortedMap;
import java.util.Collections;

import types.*;

/*
 * manages Peer lists. 
 * joining network
 * listens to NC, processes mesages concerning active peers = refreshing peer list
 * 
 * may also send ping messages - to detect inactive peers
 */
 
public class PeerManagementModul implements Module {
        /* pinging task */
        class Pinging extends TimerTask {
                
                public Pinging() {
                }
    
                @Override
                public synchronized void run() {
                        /*System.out.println("---");
                        PeerInfo[] p = data.getActivePeers();
                        for (int i = 0; i < p.length; i++)
                                System.out.println(p[i].id() + " " + p[i].ip().getHostAddress() + " " + p[i].port());
                        System.out.println("---");*/
                    
                        if (pingPosition >= actualPeers.length) { synchronized (data) {
                                /* take the discovery pong responses into account */
                                for (int i = 0; i < actualPeers.length; i++) {
                                        PeerList.IPAddress ipaddr =
                                                new PeerList.IPAddress(actualPeers[i].ip(), actualPeers[i].port());
                                        
                                        if (discoveryPongs.containsKey(ipaddr))
                                                pingReply[i] = discoveryPongs.get(ipaddr);
                                }
                                discoveryPongs.clear();
                            
                                if (round == 2) {
                                        round = 0;
                                        
                                        /* evaluate all ping responses */
                                        for (int i = 0; i < actualPeers.length; i++)
                                                if (!data.wasRecentlyDeleted(actualPeers[i]))
                                                    data.updatePeerID(actualPeers[i], pingReply[i]);
                            
                                        /* prepare new actual peers list */
                                        data.clearRecentlyDeleted();
                                        actualPeers = data.getPeerList();
                                        pingReply = new PeerID[actualPeers.length];
                                
                                        for (int i = 0; i < actualPeers.length; i++)
                                                pingReply[i] = null;
                                } else {
                                        round++;
                                }
                                
                                pingPosition = 0;
                                
                                sendingPosition += 10;
                                if (sendingPosition >= actualPeers.length)
                                        sendingPosition = 0;
                                
                                /* backup... cuz why not? */
                                data.backupPeerList();
                        } }
                        
                        /* NOTE! This is not an "else" branch */
                        /* actualPeers.length could be == 0   */
                        if (pingPosition < actualPeers.length) {
                                Message message = new Message();
                                
                                message.streamID = null;
                                message.streamClosed = false;
                                message.remotePeer = actualPeers[pingPosition];
                                message.messageType = 0x00;
                                message.sourceID = data.myPeerID();
                                message.targetID = message.remotePeer.id();
                                if (message.targetID == null) {
                                        message.targetID = new PeerID(0);
                                }
                                message.identifier = new SequenceID(pingPosition + 1);
                                
                                message.dataLength = 0;
                                message.data = new byte[0];
                                /*
                                message.dataLength = min(10, actualPeers.length - sendingPosition) * 8;
                                message.data = new byte[message.dataLength];
                                
                                // send them 10 peers from my peerlist
                                for (int i = sendingPosition; i < min(sendingPosition + 10, actualPeers.length); i++) {
                                        byte[] b_addr = actualPeers[i].ip().getAddress();
                                        byte[] b_port = Convert.intToBytes(actualPeers[i].port());
                                        
                                        for (int j = 0; j < 4; j++) {
                                                message.data[(i - sendingPosition) * 8 + j] = b_addr[j];
                                                message.data[(i - sendingPosition) * 8 + 4 + j] = b_port[j];
                                        }
                                }*/
                                        
                                try {
                                        comm.sendMessage(message);
                                        //System.out.println("Pinging.");
                                } catch (IOException e) {
                                        data.raiseError("*** Error sending ping to " +
                                                        message.remotePeer.ip().getHostAddress() +
                                                        ":" + message.remotePeer.port() + " ***");
                                }
                                
                                ++pingPosition;
                        }
                }
        }

        /* general members */
        private Communication comm;
        private DataStorage data;
        private Thread thread;
        private boolean running = true;
        private Timer timer;
        private PeerInfo[] actualPeers;
        private PeerID pingReply[];
        private int pingPosition;
        private int sendingPosition;
        private int round;
        private SortedMap<PeerList.IPAddress, PeerID> discoveryPongs;

        public PeerManagementModul() {
                this.data = DataStorage.getInstance();
                this.comm = data.getCommunicationInstance();
                
                actualPeers = data.getPeerList();
                pingReply = new PeerID[actualPeers.length];
                pingPosition = 0;
                sendingPosition = 0;
                round = 0;
                
                discoveryPongs = Collections.synchronizedSortedMap(new TreeMap<PeerList.IPAddress, PeerID>());
                
                timer = new Timer(true);
                timer.scheduleAtFixedRate(new Pinging(), 0, 2000);
        }
        
        private int min(int x, int y) {
                if (x < y) return x;
                else return y;
        }
	
        @Override
        public void run() {
                Message message;
            
                thread = Thread.currentThread();
                
                while (running) {
                        message = null;
                        try {
                                message = comm.receiveMessage(ModulType.PEER_MANAGEMENT);
                        } catch (InterruptedException e) {
                                /* nothing to do here, just to break from the blocking take. */
                        }

                        if (message == null) continue;
                        
                        /* At this place, update id for peers in the peerlist;     */
                        /* If the ID was null, it means the peer was considered    */
                        /* inactive; but if it is noticed to communicate, it means */
                        /* the peer is active; also, send keep-alives to notice    */
                        /* whether the peers didn't go inactive again.             */
                        
                        /* Well! Whatever it is, we need to get the address! */
                        if (!data.wasRecentlyDeleted(message.remotePeer))
                                data.addPeer(message.remotePeer);
                        
                        /* is it a keep-alive ping with a peerlist snippet? */
                        if (message.messageType == 0x00 && message.data != null) {
                                if (message.dataLength % 8 != 0) {
                                        data.raiseError("*** Wrong incoming ping format. ***");
                                }
                                
                                for (int i = 0; i < message.dataLength / 8; i++) {
                                        byte[] b_addr = new byte[4];
                                        byte[] b_port = new byte[4];
                                        
                                        for (int j = 0; j < 4; j++)
                                                b_addr[j] = message.data[8 * i + j];
                                        
                                        try {
                                                InetAddress addr = InetAddress.getByAddress(b_addr);
                                                int port = Convert.bytesToInt(message.data, 8 * i + 4);
                                                if (!addr.isAnyLocalAddress())
                                                        data.addPeer(new PeerInfo(null, addr, port));
                                        } catch (UnknownHostException E) {
                                                data.raiseError("*** Wrong incoming ping format. ***");
                                        }
                                }
                        }
                        
                        /* received a keep-alive pong? mark it in the pingReply[] array */
                        synchronized(data) {
                                /* keep-alive pong */
                                if (message.messageType == 0x01 && message.identifier.value() > 0 &&
                                    message.identifier.value() <= actualPeers.length &&
                                    actualPeers[message.identifier.value() - 1].equals(message.remotePeer)) {
                                
                                            pingReply[message.identifier.value() - 1] = message.sourceID;
                                            
                                /* discovery pong */
                                } else if (message.messageType == 0x01) {
                                        discoveryPongs.put(new PeerList.IPAddress(message.remotePeer.ip(),
                                                                                  message.remotePeer.port()),
                                                           message.remotePeer.id());
                                }
                        }
                        
                        /* received a ping? if it's correct, will pong; if not, will send a ping of my own */
                        if (message.messageType == 0x00) {
                                Message response = message.deepCopy();
                                
                                if (response.data == null) {
                                        response.dataLength = 0;
                                        response.data = new byte[0];
                                }
                                
                                response.sourceID = data.myPeerID();
                                response.targetID = message.sourceID;
                                
                                if (message.targetID.equals(data.myPeerID())) {
                                        // I'm the real target; replying with a pong
                                        response.messageType = 0x01;
                                } else {
                                        // I'm not the real target; better send 'em my real ID
                                        response.messageType = 0x00;
                                        response.identifier = new SequenceID(0);
                                }
                                
                                try {
                                        comm.sendMessage(response);
                                } catch (IOException e) {
                                        data.raiseError("*** Error sending message to " +
                                                        message.remotePeer.ip().getHostAddress() +
                                                        ":" + message.remotePeer.port() + " ***");
                                }
                        }

                }
        }
	
	@Override
        public void exit() {
                running = false;
                timer.cancel();
                thread.interrupt();
        }
}
