package modules;

import java.io.IOException;
import java.net.DatagramPacket;
import java.net.DatagramSocket;
import java.util.HashMap;
import java.util.concurrent.ArrayBlockingQueue;

import types.Message;
import types.ModulType;
import types.PeerInfo;

public class UDPListener extends NetworkListener {
	//TODO remove constrain
	private int MAX_UDP_DATAGRAM_LENGTH = 65536;
	
	private DatagramSocket udpSocket;
	private boolean exit = false;
	
	public UDPListener(DataStorage dataStorage,
					   HashMap<ModulType, ArrayBlockingQueue<Message> > msgStorage, 
					   int port) 
	throws IOException 
	{
		this.dataStorage = dataStorage;
		this.msgStorage = msgStorage;
		udpSocket = new DatagramSocket(port);
	}
	
	public DatagramSocket udpSocket() {
		return udpSocket;
	}
	
	@Override
	public void exit() {
		exit = true;
		udpSocket.close();
	}

	@Override
	public void run() {
		while (!exit) {
			byte[] buf = new byte[MAX_UDP_DATAGRAM_LENGTH];
			DatagramPacket udpPacket = new DatagramPacket(buf, buf.length);
			
			try {
				udpSocket.receive(udpPacket);
			} catch (IOException e) {
				dataStorage.debugMessage("IOException while recieving message");
				continue;
			}
			
			Message msg = Message.messageFromBytes(buf, udpPacket.getLength());
			if (msg == null) {
				dataStorage.debugMessage("unable to convert message from bytes");
				continue;
			}
			if (msg.messageType == 0x04 || msg.messageType == 0x05 || msg.messageType == 0x06 
			  || msg.messageType == 0x14 || msg.messageType == 0x15 || msg.messageType == 0x16) 
			 {
				dataStorage.debugMessage("message type not meant for UDP");
				continue;
			}
			msg.streamID = null;
			msg.remotePeer = new PeerInfo(msg.sourceID, udpPacket.getAddress(), udpPacket.getPort());
			processMessage(msg);
		}
		
		udpSocket.close();
	}
	
}
