package modules;

import java.io.IOException;
import java.util.HashMap;

import auxiliary.Convert;
import auxiliary.SHA1;

import types.*;

//!TODO synchronized
/*
 * handles uploading shared files, responds to queries
 * 
 * UM is listening to NC, when query comes, creates UploadThread
 * UM forwards other messages form NC to responsible UT, and otherwise
 * (UT <-> UM <-> NC)
 */

public class UploadModul implements Module {
	
	//REFACTOR: redundancy of declarations through modules
    private Communication comm;
    private DataStorage data;
    
    private boolean running = true;
    private Thread myThread;
	
    private HashMap <SequenceID, UploadThread> threads;
    
	public UploadModul() {
		this.data = DataStorage.getInstance();
		this.comm = data.getCommunicationInstance();
		this.threads = new HashMap<SequenceID, UploadThread>();
	}
	
	//!TODO thread management
	//public UploadThread[] activeTransfers() {return null; }

	//Same as in DownloadModul
	public void finishUpload(SequenceID id){
		if(id == null)
			return;
		if(!this.threads.containsKey(id))
			return;
		
		//All what is necessary in destruction will be handled in that concrete thread
		this.threads.get(id).exit();
		this.threads.remove(id);
	}
	
	// called by UT -- sends message to DownloadModul
	public void sendMessage(Message message){
		if(!message.streamClosed){
			message.encode();
			message.dataLength = ((message.data == null) ? 0 : message.data.length);
			DownloadModul.messageTypeForDownload(message);
		}
		try {
			this.comm.sendMessage(message);
		} catch (IOException e) {
			data.raiseError("Unable to send message");
		}
	}
	
	private void serializeMessage(Message message){
		//DataStorage.getInstance().debugMessage("Upload modul serialize message. SequenceID: " + message.identifier + " type: 0x" + Convert.byteToAlphaHex(message.messageType));
		//System.out.println("\nUpload modul serialize message:\n" + message.toString());
		
		//!TODO -- NetworkCommunication sends streamClosed message just for downloadModul...
		if(message.streamClosed){
			//DataStorage.getInstance().debugMessage("UploadThread SeqId " + message.identifier + " closed because streamClosed = true.");
			//Finish UploadThread (identified with streamId) 
			//Notify communication, that the conversation was canceled. 
			this.sendMessage(message);
			return;
		}
		
		DownloadModul.messageTypeForParse(message);
		DownloadMessage.parse(message);
		
		
		//---------------INIT MESSAGE ------------
		if(message.representation instanceof DownloadMessage.InitMessage){
			DownloadMessage.InitMessage data = (DownloadMessage.InitMessage) message.representation;

			//For example it exists no more (DataStorage.SharedDatabase will respond)
			if(null == DataStorage.getInstance().SharedFiles().getFilenameByChecksum(data.init_sha)){
				DataStorage.getInstance().debugMessage("Shared file: " + data.init_sha + " not contained.");
				data.init_type = DownloadMessage.InitType.CANCEL;
			}
			//Regenerate SequenceID
			else if(this.threads.containsKey(message.identifier)){
				data.init_type = DownloadMessage.InitType.REGEN;
			}
			else{
				UploadThread upload = new UploadThread(this, data.init_sha, message.getDefaultReply());
				this.threads.put(message.identifier, upload);
				(new Thread(upload)).start();
				
				data.init_type = DownloadMessage.InitType.OK;
				data.file_size = upload.fileSize();
			}
		}
		//---------------CANCEL MESSAGE --------------------
		else if(message.representation instanceof DownloadMessage.CancelMessage){
			DownloadMessage.CancelMessage data = (DownloadMessage.CancelMessage) message.representation;
			DataStorage.getInstance().debugMessage("Upload thread SeqID " + message.identifier + " terminated because " + data.cancelReason());
			this.finishUpload(message.identifier);
			
			//If this message received, then on the other side was the communication also closed. 
			return;
		}
		//----------------------PART MESSAGE--------------
		else if(message.representation instanceof DownloadMessage.PartMessage){
			if(!this.threads.containsKey(message.identifier)){
				DataStorage.getInstance().debugMessage("Not found thread with seq_id="+message.identifier+" at UM.serializeMessage.PartMessage");
				return;
			}
			//So from now we can assume that such a thread exists. 
			UploadThread thread = this.threads.get(message.identifier);
			DownloadMessage.PartMessage data = (DownloadMessage.PartMessage) message.representation;
			
			//Tells to the concrete UploadThread that part_id should be uploaded. 
			thread.addToSend(data.part_id);
			return;
		}else{
			DataStorage.getInstance().debugMessage("Unhandled message in UploadModul: " + message.toString());
			return;
		}
		Message reply = message.getDefaultReply();
		reply.representation = message.representation;
		this.sendMessage(reply);
	}
	
    @Override
    public void run() {
    		myThread = Thread.currentThread();
            while (running) {
            	Message message=null;
                try {
                    message = comm.receiveMessage(ModulType.UPLOAD);
                } catch (InterruptedException e) {
                	//data.debugMessage("UM interrupted");
                	//Nothing to do, if really interrupted, then it will be closed because running = false.
                	continue;
                }
				this.serializeMessage(message);
            }
    }

    @Override
    public void exit() {
		//!TODO remove
		data.debugMessage("UM exit()");
		
        //Stop all threads running
        for(UploadThread ut : this.threads.values()){
        	ut.exit();
        }
        
        running = false;
        if (myThread != null) myThread.interrupt();
    }
    
    // for stats
    public int activeThread() {
    	return threads.size();
    }
}
