package modules;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.RandomAccessFile;
import java.util.concurrent.ArrayBlockingQueue;

import auxiliary.SHA1;

import types.DownloadedFileDescriptor;
import types.Message;
import types.SequenceID;
import types.SharedFileDescriptor;

//!TODO synchronized
/*
 * uploads file - sending messages to UM, which forwards them to NC
 * also receiving confirming messages
 */

public class UploadThread implements Runnable {
	private static int blocking_queue_size = 1024;
	private RandomAccessFile in = null; 
	private ArrayBlockingQueue<Integer> partsToSend = null; 
	private Message default_reply = null; 
	private UploadModul parent = null;
	
	private boolean finished = false;

	public SequenceID getSequenceId(){
		if(default_reply != null){
			return default_reply.identifier;
		}
		return null;
	}
	
	public UploadThread(UploadModul um, SHA1 checksum, Message reply) {
		this.partsToSend = new ArrayBlockingQueue<Integer>(UploadThread.blocking_queue_size);
		this.default_reply = reply;
		this.parent = um;
		
		SharedFileDescriptor sfd = DataStorage.getInstance().SharedFiles().getDescriptorByChecksum(checksum);
		if(sfd == null){
			DataStorage.getInstance().debugMessage("UploadThread " + this.default_reply.identifier + " not initialized - uploaded file not found.");
			this.finished = true;
			return; 
		}
		
		//Creates file handler for upload
		try {
			this.in = new RandomAccessFile(
				sfd.getFilepath(), 
				"r"
			);
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			// !TODO Kill this thread
			DataStorage.getInstance().debugMessage("");
			e.printStackTrace();
		}
	}
	
	//The last part (which don`t have to be long as part_size)
	//--because of message consistency, I will return part_size long byte array
	//--It is resolved in DownloadedFileDescriptor.addPart()
	private byte[] getPart(int part_id){
		DataStorage.getInstance().debugMessage("getPart :" + part_id);
		byte[] result = new byte[DownloadedFileDescriptor.part_size];
		try {
			this.in.seek(((long)DownloadedFileDescriptor.part_size) * ((long)part_id));
			this.in.read(result);
		} catch (IOException e) {
			// TODO Auto-generated catch block
			// --error handling
			return null;
		}
		return result;
	}
	
	public long fileSize(){
		//!TODO I don`t understand why this this function throws Exception
		try {
			return this.in.length();
		} catch (IOException e) {
			return 0;
		}
	}
	
	@Override
	public void run() {
		DownloadMessage factory = new DownloadMessage();
		
		while(!finished){
			//Sending requested parts
			//--other replies are responded from UploadModul (are quite trivial)
			
			int part_id; 
			try {
				part_id = this.partsToSend.take();
			} catch (InterruptedException e) {
				continue;
			}
			
			DownloadMessage.PartMessage part = factory.new PartMessage();
			part.part_id = part_id;
			part.part_data = this.getPart(part_id);
			
			this.default_reply.representation = part;
			this.parent.sendMessage(default_reply);
		}
		//Remove from UploadModul.threads
		this.parent.finishUpload(this.getSequenceId());
	}

	public void addToSend(int part_id) {
		//!TODO remove
		DataStorage.getInstance().debugMessage("UT add part to up: " + part_id);
		this.partsToSend.add(part_id);
	}

	public void exit(){
		this.finished = true;
	}
}
