package terminalui;

import java.util.ArrayList;
import java.util.TreeSet;

public class Command extends TerminalIO {
	
	String[] args;
	TreeSet<String> flags;
	
	public CmdReturn execute(String[] args) {
		return CmdReturn.INVALID;
	}
	
	public void help() {
		TEXT.outln("No such command");
	}
	
	public CmdReturn displayHelp() {
		if (flags.size() != 1) return CmdReturn.INVALID_SYNTAX;
		if (args.length != 0) return CmdReturn.INVALID_SYNTAX;
			
		help();
		
		return CmdReturn.SUCCESS;
	}
	
	public static Command commandFromString(String command) {
		Command cmd;
		
		if ("exit".equals(command)) cmd = new ExitCommand();
		else if ("help".equals(command)) cmd = new HelpCommand();
		else if ("status".equals(command)) cmd = new StatusCommand();
		else if ("search".equals(command)) cmd = new SearchCommand();
		else if ("download".equals(command)) cmd = new DownloadCommand();
		else if ("share".equals(command)) cmd = new ShareCommand();
		else if ("peer".equals(command)) cmd = new PeerCommand();
		else if ("debug".equals(command)) cmd = new DebugCommand();
		else cmd = new InvalidCommand();
		
		return cmd;
	}
	
	public static String[] commandsToStringArray() {
		ArrayList<String> cmd = new ArrayList<String>();
		cmd.add("debug");
		cmd.add("help");
		cmd.add("exit");
		cmd.add("download");
		cmd.add("peer");
		cmd.add("search");
		cmd.add("share");
		cmd.add("status");
		String[] result = new String[cmd.size()];
		result = cmd.toArray(result);
		return result;	
	}
	
	protected boolean parseArgs(String[] commandArgs, String[] supportedFlags) {
		flags = new TreeSet<String>();
		TreeSet<String> sFlags = new TreeSet<String>();
			for (String flag: supportedFlags) sFlags.add(flag);
		ArrayList<String> args_ = new ArrayList<String>();
		
		boolean noflags = false;
		boolean nocmd = false;
		for (int i=1; i<commandArgs.length; i++) {
			String str = commandArgs[i];	
			if (sFlags.contains(str) || str.charAt(0) == '-') {
				if (str.charAt(0) == '-') {
					if (str.length() == 1) return false;
					if (noflags) return false;
					nocmd = true;
					for (int j=1; j<str.length(); j++){
						String flag = "-" + str.charAt(j);
						if (!sFlags.contains(flag)) return false;
						flags.add(flag);
					}
				}
				else {
					if (nocmd) args_.add(str);
					else flags.add(str);
					nocmd = true;
				}
			}
			else {
				nocmd = true;
				noflags = true;
				if (str.charAt(0) == '"' && str.charAt(str.length()-1) == '"')
					args_.add(str.substring(1, str.length()-1));
				else 
					args_.add(str);
			}
		}
		
		args = new String[args_.size()];
		args = args_.toArray(args);
		return true;
	}
}
