package terminalui;


import modules.DataStorage;

public class DebugCommand extends Command{
	@Override
	public CmdReturn execute(String[] args) {
		String[] supportedFlags = {"on", "off", "help"};
		if (!parseArgs(args, supportedFlags)) return CmdReturn.INVALID_SYNTAX;
		
		if (flags.contains("help")) return displayHelp();
		if (flags.size() != 0) return setDebug();
		
		return debugStatus();
	}
	
	@Override
	public void help() {
		TEXT.out("    Used for switching debug mode.\n");
		TEXT.out("USAGE\n"); 
		TEXT.out("    debug <subcommand> \n");
		TEXT.out("SUBCOMMANDS\n");
		TEXT.out("    help     displays help for this command\n");
		TEXT.out("    on       turn on debug mode\n");
		TEXT.out("    off      turn off debug mode\n");
		TEXT.out("\n    Without subcommand displays debug mode.\n");
		TEXT.out("OPTIONS\n");
		TEXT.out("    none\n");
	}
	
	private CmdReturn debugStatus() {
		if (args.length != 0) return CmdReturn.INVALID_SYNTAX;
		if (flags.size() != 0) return CmdReturn.INVALID_SYNTAX;
		
		if (DataStorage.getInstance().isDebugActive())
			TEXT.out("debug active\n");
		else TEXT.out("debug inactive\n");
		
		return CmdReturn.SUCCESS;
	}

	private CmdReturn setDebug() {
		if (args.length != 0) return CmdReturn.INVALID_SYNTAX;
		if (flags.size() != 1) return CmdReturn.INVALID_SYNTAX;
		
		if (flags.contains("on"))
			DataStorage.getInstance().setDebug(true);
		else if (flags.contains("off"))
			DataStorage.getInstance().setDebug(false);
		else
			return CmdReturn.INVALID_SYNTAX;
		
		return CmdReturn.SUCCESS;
	}

}
