package terminalui;

import java.util.ArrayList;
import java.util.TreeSet;

import modules.DataStorage;
import modules.SearchModul;
import terminalui.TerminalIO.FSInputContainer;
import types.SearchQuery;
import types.SearchResult;
import types.SearchResultReceiver;

public class SearchCommand extends Command implements FSInputContainer, SearchResultReceiver{

	@Override
	public CmdReturn execute(String[] args) {
		String[] supportedFlags = {"results", "help"};
		if (!parseArgs(args, supportedFlags)) return CmdReturn.INVALID_SYNTAX;
		
		if (flags.contains("help")) return displayHelp();
		if (flags.contains("results")) return searchResults();
		
		return searchKeywords();
	}
	
	
	@Override
	public void help() {
		TEXT.out("    Used for searching files in TazUP network.\n");
		TEXT.out("USAGE\n"); 
		TEXT.out("    search [<subcommand>] [<options>] [<keyword> [...]]\n");
		TEXT.out("SUBCOMMANDS\n");
		TEXT.out("    help     displays help for this command\n");
		TEXT.out("    results  displays actual search result.\n");
		TEXT.out("\n    Without subcommand searches for files in network matching keywords specified.\n");
		TEXT.out("OPTIONS\n");
		TEXT.out("    none\n");
		TEXT.out("KEYWORDS\n");
		TEXT.out("    Without subcommand at least one keyword must be specified.\n");
	}
	
	private CmdReturn searchResults() {
		if (args.length != 0) return CmdReturn.INVALID_SYNTAX;
			
		startFullScreen(this);
		
		return CmdReturn.SUCCESS;
	}
	
	private CmdReturn searchKeywords() {
		if (this.args.length == 0) return CmdReturn.MISSING_ARGUMENT;
		
		SearchQuery sq = new SearchQuery();
		sq.keywords = new TreeSet<String>();
		for (String s: args) sq.keywords.add(s);
		
		SearchModul sm = DataStorage.getInstance().getSearchInstance();
		
		sm.search(sq);
		
		TEXT.out("search initiated..\n");
		
		return CmdReturn.SUCCESS;
	}


	FSInput[] input = null;
	@Override
	public FSInput[] getData() {
		
		if (input != null) return input;
		
		ArrayList<FSInput> in = new ArrayList<FSInput>();
		DataStorage ds = DataStorage.getInstance();
		SearchResult[] results = ds.getSearchResultsSortedByTime();
	
		//
		in.add(new FSInput("Search result", FSColor.GREEN, 0, 0));
		in.add(new FSInput("Results: ", FSColor.SYSTEM, 0, 1));
		in.add(new FSInput("" + results.length, FSColor.TEXT, 15, 1));
		
		in.add(new FSInput("No.", FSColor.SYSTEM, 0, 3));
		in.add(new FSInput("File name", FSColor.SYSTEM, 10, 3));
		in.add(new FSInput("File size", FSColor.SYSTEM, 30, 3));
		in.add(new FSInput("PeerID", FSColor.SYSTEM, 50, 3));
		in.add(new FSInput("File Hash", FSColor.SYSTEM, 70, 3));
		
		int startline = 4;
		for (int i=0; i<results.length; i++) {
			SearchResult sr = results[i];
			
			in.add(new FSInput(""+i, FSColor.TEXT, 0, startline+i));
			in.add(new FSInput(sr.filename(), FSColor.TEXT, 10, startline+i));
			in.add(new FSInput(""+sr.filesize()+" B", FSColor.TEXT, 30, startline+i));
			in.add(new FSInput(sr.peer().toString(), FSColor.TEXT, 50, startline+i));
			in.add(new FSInput(sr.checksum().toString(), FSColor.TEXT, 70, startline+i));
		}
		//
		input = new FSInput[in.size()];
		input = in.toArray(input);
		return input;
	}


	@Override
	public void receiveSearchMessage() {
		// TODO Auto-generated method stub
		
	}
}
