package terminalui;

import modules.DataStorage;

public class StatusCommand extends Command{

	@Override
	public CmdReturn execute(String[] args) {
		String[] supportedFlags = {"-d", "-p", "-a", "-f", "-s", "help"};
		if (!parseArgs(args, supportedFlags)) return CmdReturn.INVALID_SYNTAX;
		
		if (flags.contains("help")) return displayHelp();
		if (flags.contains("-p")) return showPeerStatus();
		if (flags.contains("-d")) return showDownloadStatus();
		if (flags.contains("-f")) return showSharedFiles();
		if (flags.contains("-s")) return showSearchResult();
		
		return showGeneralStatus();
	}

	@Override
	public void help() {
		TEXT.out("    Used for displaying status of TazUP client.\n");
		TEXT.out("USAGE\n"); 
		TEXT.out("    status [<options>]\n");
		TEXT.out("SUBCOMMANDS\n");
		TEXT.out("    help     displays help for this command\n");
		TEXT.out("OPTIONS\n");
		TEXT.out("    -p       show active peers. Same as peer\n");
		TEXT.out("    -d       show download status. Same as download status\n");
		TEXT.out("    -f       show shared files. Same as share\n");
		TEXT.out("    -s       show search results. Same as search results\n");
		TEXT.out("    -a       show all. Default in all except -p\n");
	}

	private CmdReturn showGeneralStatus() {
		if (args.length != 0) return CmdReturn.INVALID_SYNTAX;
		
		DataStorage ds = DataStorage.getInstance();
		
		TEXT.out("Active peers:     " + ds.getActivePeerCount() + " (" + ds.getPeerCount() + ")\n");
		TEXT.out("Shared files:     " + ds.SharedFiles().size() + "\n");
		TEXT.out("Active downloads: " + ds.activeDownloads() + "\n");
		TEXT.out("Active uploads:   " + ds.activeUploads() + "\n");
		
		return CmdReturn.SUCCESS;
	}
	
	private CmdReturn showDownloadStatus() {
		if (args.length != 0) return CmdReturn.INVALID_SYNTAX;
		if (flags.size() != 1 && !flags.contains("-a")) return CmdReturn.INVALID_SYNTAX; 
		
		Command cmd = new DownloadCommand();
		String[] arg = new String[2];
		arg[0] = "download";
		arg[1] = "status";
		return cmd.execute(arg);
	}

	private CmdReturn showSharedFiles() {
		if (args.length != 0) return CmdReturn.INVALID_SYNTAX;
		if (flags.size() != 1 && !flags.contains("-a")) return CmdReturn.INVALID_SYNTAX;  
		
		Command cmd = new ShareCommand();
		String[] arg = new String[1];
		arg[0] = "share";
		return cmd.execute(arg);
	}

	private CmdReturn showSearchResult() {
		if (args.length != 0) return CmdReturn.INVALID_SYNTAX;
		if (flags.size() != 1 && !flags.contains("-a")) return CmdReturn.INVALID_SYNTAX; 
		
		Command cmd = new SearchCommand();
		String[] arg = new String[2];
		arg[0] = "search";
		arg[1] = "results";
		return cmd.execute(arg);
	}
	
	private CmdReturn showPeerStatus() {
		if (args.length != 0) return CmdReturn.INVALID_SYNTAX;
		if (flags.size() != 1 && !flags.contains("-a")) return CmdReturn.INVALID_SYNTAX; 
		
		Command cmd = new PeerCommand();
		String[] arg = new String[flags.size()];
		arg[0] = "peer";
		if (flags.contains("-a")) arg[1] = "-a";
		return cmd.execute(arg);
	}
}
