package terminalui;

import jlibs.core.lang.Ansi;
import jcurses.system.*;


public class TerminalIO {
	protected static Ansi TEXT = new Ansi(null, null, null);
	protected static Ansi GREEN = new Ansi(null, Ansi.Color.GREEN, null);
	protected static Ansi SYSTEM = new Ansi(null, Ansi.Color.YELLOW, null);
	protected static Ansi ERROR = new Ansi(null, Ansi.Color.RED, null);
	
	protected static boolean compatibilityCheck() {
		if (!Ansi.SUPPORTED) return false;
		return true;
	}
	
	
	
	private FSInputContainer container = null;
	private boolean ended;
	private int offY;
	private int minOffY;
	private Thread thread;
	
	protected void startFullScreen(FSInputContainer container) {
		this.container = container;
		this.thread = Thread.currentThread();
		offY = 0;
		minOffY = 1;
		TerminalUI.fullscreenMode = true;
		Toolkit.init();
		ended = false;
		
		Thread dispatch = new Thread(new EventDispatch());
		dispatch.start();
		
		while (!ended) {
			diplayData();
			try {
				Thread.sleep(500);
			} catch (InterruptedException e) {}
		}
	}
	
	protected void endFullScreen() {
		Toolkit.shutdown();
		TerminalUI.fullscreenMode = false;
		ended = true;
		if (thread != null) thread.interrupt();
	}
	
	private void diplayData() {
			FSInput[] input = container.getData();
			//
			Toolkit.clearScreen(new CharColor(CharColor.BLACK, CharColor.WHITE));
			for (int i=0; i<input.length; i++) {
				CharColor c = new CharColor(CharColor.BLACK, CharColor.WHITE);
				switch (input[i].color) {
					case GREEN: c = new CharColor(CharColor.BLACK, CharColor.GREEN); break;
					case SYSTEM: c = new CharColor(CharColor.BLACK, CharColor.YELLOW); break;
					case ERROR: c = new CharColor(CharColor.BLACK, CharColor.RED); break;
					case TEXT: c = new CharColor(CharColor.BLACK, CharColor.WHITE); break;
				}
				
				int width = Toolkit.getScreenWidth() - input[i].col;
				int height = (input[i].text.length() / width) + 1;
				if (input[i].row + offY < Toolkit.getScreenHeight() &&
					input[i].row + offY >= 0 &&	
					input[i].col < Toolkit.getScreenWidth() &&
					input[i].col >= 0)
					Toolkit.printString(input[i].text, input[i].col, input[i].row + offY, width, height, c);
				
				if (Toolkit.getScreenHeight() - height - input[i].row < minOffY)
					minOffY = Toolkit.getScreenHeight() - height - input[i].row;
			}
	}
	
	//-------------
	protected interface FSInputContainer {
		FSInput[] getData();
	}
	
	protected class FSInput {
		public String text;
		public FSColor color;
		public int col, row;
		
		public FSInput(String text, FSColor color, int col, int row) {
			this.text = text;
			this.color = color;
			this.col = col;
			this.row = row;
		}
	}
	
	protected enum  FSColor {
		GREEN, SYSTEM, ERROR, TEXT
	}
	
	private class EventDispatch implements Runnable {
		@Override
		public void run() {
			while (true) {
				InputChar c = Toolkit.readCharacter();
				if (!c.isSpecialCode() && c.getCharacter() == 'q') break;
				if (c.getCode() == 259) offY++;					// up arrow
				else if (c.getCode() == 258) offY--;			// down arrow
				
				if (offY > 0) offY = 0;
				if (offY < minOffY) offY = minOffY;
			}
			endFullScreen();
		}
		
	}
}
