package terminalui;

import java.util.ArrayList;
import java.util.Scanner;

import jcurses.system.Toolkit;

import modules.DataStorage;
import modules.DebugCollector.DebugCollectorObserver;
import modules.ErrorCollector.ErrorCollectorObserver;

public class TerminalUI extends TerminalIO implements Runnable, ErrorCollectorObserver, DebugCollectorObserver {
	private boolean exit = false;
	
	protected static boolean fullscreenMode = false;
	
	public TerminalUI() {
		DataStorage.getInstance().addErrorCollectorObserver(this);
		DataStorage.getInstance().addDebugMessagesObserver(this);
	}
	
	public void run() {	
		if (!compatibilityCheck()) {
			ERROR.out("Warning: UI may not be displayed corectly");
		}
	
		startupInfo();
		commandLoop();
	}
	
	private void startupInfo() {
		GREEN.outln("Starting TazUP client, version 1.0");
	}
	
	private void commandLoop() {
		while(!exit) {
			String command = getCommand();
			String[] parsedCommand = parseCommand(command);
			proccessCommand(parsedCommand);
		}
	}
	
	private String getCommand() {
		SYSTEM.out("TazUP > ");
		Scanner in = new Scanner(System.in);
		return in.nextLine();
	}
	
	private String[] parseCommand(String command) {
		Scanner sc = new Scanner(command);
		ArrayList<String> parsed = new ArrayList<String>();
		while (sc.hasNext()) 
			parsed.add(sc.next());
		String[] result = new String[parsed.size()];
		parsed.toArray(result);
		return result;
	}
	
	private void proccessCommand(String[] args) {
		if (args.length == 0) return;
		
		Command cmd = Command.commandFromString(args[0]);
		
		switch(cmd.execute(args)) {
			case SUCCESS:
				
				break;
			case MISSING_ARGUMENT:
				TEXT.outln("Missing argument for command \"" + args[0] + "\"");
				break;
			case EXIT:
				exit = true;
				break;
			case INVALID:
				TEXT.outln("Command \"" + args[0] + "\" not recognized");
				break;
			case INVALID_SYNTAX:
				TEXT.outln("Invalid syntax");
				break;
		}
	}

	@Override
	public void errorRaised(String description) {
		if (!fullscreenMode) {
			ERROR.out("Internal error: " + description + "\n");
		}
	}
	@Override
	public void messageThrown(String description) {
		if (!fullscreenMode && DataStorage.getInstance().isDebugActive()){		
			TEXT.out("Debug message: " + description +"\n");
		}
	}
	@Override
	protected void finalize() {
		Toolkit.shutdown();
	}
	
}
