package types;

import java.net.InetAddress;
import java.net.UnknownHostException;
import java.nio.charset.Charset;
import java.util.ArrayList;
import java.util.TreeSet;

import auxiliary.*;
import modules.*;
/*
 * used to describe search query from user
 */

public class SearchQuery {
	
	public static int HEADER_SIZE = 16; 

	public TreeSet<String> keywords;
	public PeerInfo initiator;
	public int ttl;

	public static SearchQuery parseKeywords(String str) {
		if (str == null) {
			DataStorage.getInstance().raiseError("*** Wrong keywords to parse***");
			return null;
		}
		
		SearchQuery ret = new SearchQuery();
		
		ret.keywords = new TreeSet<String>();
		
		for (String s : str.split(" "))
			ret.keywords.add(s);
		
		return ret;
	}
	
	public byte[] getBytes() {	
		if (initiator == null) return null;
		if (initiator.id() == null) return null;
		if (initiator.ip() == null) return null;
		
		ArrayList<Byte> bytes = new ArrayList<Byte>();
		
		byte[] ttlb = Convert.intToBytes(ttl);
		for (byte b: ttlb) bytes.add(b);
		
		byte[] id = initiator.id().getBytes();
		for(byte b: id) bytes.add(b);
	
		byte[] ip = initiator.ip().getAddress();
		for (byte b: ip) bytes.add(b);
	
		byte[] port = Convert.intToBytes(initiator.port());
		for (byte b: port) bytes.add(b);
		
		if (keywords != null) {
			for (String s: keywords) {
				s += ' ';
				for (byte b: s.getBytes(Charset.forName("UTF-8"))) bytes.add(b);				
			}
		}
		
		byte[] result = new byte[bytes.size()];
		for (int i=0; i<bytes.size(); i++)
			result[i] = bytes.get(i);
		
		return result;
	}
	
	public static SearchQuery querryWithBytes(byte[] bytes) {
		if (bytes == null) return null;
		if (bytes.length < HEADER_SIZE) return null;
		
		SearchQuery sq = new SearchQuery();
		
		sq.ttl = Convert.bytesToInt(bytes, 0);
		
		PeerID id = PeerID.peerIDfromBytes(bytes, 4);
		
		byte[] ipb = new byte[4];
		for (int i=0; i<4; i++) ipb[i] = bytes[i+8];
		InetAddress ip;
		try {
			ip = InetAddress.getByAddress(ipb);
		} catch (UnknownHostException e) {
			return null;
		}
		int port = Convert.bytesToInt(bytes, 12);
		
		sq.initiator = new PeerInfo(id, ip, port);
		
		if (bytes.length == HEADER_SIZE) return sq;
		
		sq.keywords = new TreeSet<String>();
		byte[] kb = new byte[bytes.length - HEADER_SIZE];
		for (int i=HEADER_SIZE; i<bytes.length; i++) kb[i-HEADER_SIZE] = bytes[i];
		String keys = new String(kb, Charset.forName("UTF-8"));
		
		for (String s: keys.split(" ")) sq.keywords.add(s);
		
		return sq;
	}
	
}
