package tests;

//Test all what is done to the FileSystem in the downloadModul


import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.RandomAccessFile;
import java.util.Arrays;
import auxiliary.SHA1;
import types.DownloadedFileDescriptor;
import junit.framework.TestCase;

public class DownloadedFileDescriptorTest extends TestCase {
		//Loads data from .bck
		//Saves data to .tdp (downloaded parts)
		//Parts data to .tfp (file parts)
		//Destination file file0.txt / file1.txt
	
		DownloadedFileDescriptor file0;
		DownloadedFileDescriptor file1;
		long file0size;
		long file1size;
		String file0path = "share/file0.txt";
		String file1path = "share/file1.txt";
		
		public static RandomAccessFile openFile(String filepath){
			try {
				return new RandomAccessFile(filepath, "r");
			} catch (FileNotFoundException e) {
				e.printStackTrace();
				fail();
				return null;
			}
		}
		public static boolean fileExists(String filepath){
			return (new File(filepath)).exists();
		}
		public static long fileSize(String filepath){
			try {
				return (new RandomAccessFile(filepath, "r")).length();
			} catch (Exception e){
				return -1;
			}
		}
		public static byte[] getFileBytes(String filepath){
			byte[] b = new byte[(int) fileSize(filepath)];
			try {
				openFile(filepath).read(b);
			} catch (IOException e) {
				e.printStackTrace();
				fail();
				return null;
			}
			return b;
		}
		public static boolean filesIsContentSame(String filepath1, String filepath2){
			return Arrays.equals(getFileBytes(filepath1), getFileBytes(filepath2));
		}
		public static boolean fileCopy(String filepath1, String filepath2){
		    File inputFile = new File(filepath1);
		    File outputFile = new File(filepath2);

		    try {
			    FileReader in = new FileReader(inputFile);
			    FileWriter out = new FileWriter(outputFile);
			    int c;
				while ((c = in.read()) != -1)
					 out.write(c);
			    in.close();
			    out.close();
			} catch (IOException e) {
				e.printStackTrace();
				return false;
			}
			return true;
		}
		
		public DownloadedFileDescriptorTest() {
			System.out.println("Constructor");
			
			fileCopy(file0path + ".bck", file0path);
			fileCopy(file1path + ".bck", file1path);
			file0 = new DownloadedFileDescriptor("file0.txt", "share/", SHA1.getFileHash(file0path));
			file1 = new DownloadedFileDescriptor("file1.txt", "share/", SHA1.getFileHash(file1path));
			file0size = fileSize(file0path);
			file1size = fileSize(file1path);
		}
		
		public void testFilesIsContentSame(){
			if(!filesIsContentSame(file0path, file0path))
				fail();
		}
		
		public void testFilepath(){
			if(!file0.getFilepath().equals(file0path)) fail();
			if(!file1.getFilepath().equals(file1path)) fail();
		}
		
		public void testGenerateNotExistingFilepath(){
			try {
				(new File("share/test")).createNewFile();
				(new File("share/test.txt.tzp")).createNewFile();
				
				if(!"share/testdaco".equals(DownloadedFileDescriptor.generateNotExistingFilepath("share/testdaco"))) fail();
				
				System.out.println(DownloadedFileDescriptor.generateNotExistingFilepath("share/test"));
				if(!"share/test(2)".equals(DownloadedFileDescriptor.generateNotExistingFilepath("share/test"))) fail();
				
				System.out.println(DownloadedFileDescriptor.generateNotExistingFilepath("share/test.txt.tzp"));
				if(!"share/test.txt(2).tzp".equals(DownloadedFileDescriptor.generateNotExistingFilepath("share/test.txt.tzp"))) fail();
				
				(new File("share/test.txt(2).tzp")).createNewFile();
				if(!"share/test.txt(3).tzp".equals(DownloadedFileDescriptor.generateNotExistingFilepath("share/test.txt.tzp"))) fail();
				
				(new File("share/test")).delete();
				(new File("share/test.txt.tzp")).delete();
				(new File("share/test.txt(2).tzp")).delete();
			} catch (IOException e) {
				fail();
			}
		}
		
		private void addPart(DownloadedFileDescriptor dfd, int part_id, byte[] data){
			try {
				dfd.addPart(part_id, data);
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
		
		public void testMissingParts(){
			if(!file1.allocate(file1size))
				fail();
			byte[] data = new byte[DownloadedFileDescriptor.part_size];
			
			this.addPart(file1, 1, data);
			if(file1.getMissingPart() != 0) fail();
			
			this.addPart(file1, 0, data);
			if(file1.getMissingPart() != 2) fail();
			
			this.addPart(file1, 2, data);
			if(file1.getMissingPart() != 3) fail();
			
			for(int i=0; i<file1.getPartsCount() - 4; i++){
				file1.getMissingPart();
			}
			if(file1.getMissingPart() != file1.getPartsCount() - 1);
			
			this.addPart(file1, 3, data);
			this.addPart(file1, 4, data);
			this.addPart(file1, 5, data);
			this.addPart(file1, 6, data);
			if(file1.getMissingPart() != 7) fail();
		}
		
		public void testAllocateFile0(){
			if(!file0.allocate(file0size)) fail(); 
			if(!fileExists(file0path)) fail();
			if(file0size != fileSize(file0path)) fail();
		}
		
		public void testPartsCount(){
			if(!file0.allocate(file0size));
			
			System.out.println("File0 part count: " + file0.getPartsCount());
			System.out.println("File1 part count: " + file1.getPartsCount());
		}
		
		public void testAllocateFile1(){
			if(file1.allocate(file1size)) fail();
			if(!fileExists(file1path)) fail();
			if(file1size != fileSize(file1path)) fail();
		}
	
		public void testAddPartFile0(){
			helpTestAddPart(file0);
		}
		public void testAddPartFile1(){
			helpTestAddPart(file1);
		}
		public void helpTestAddPart(DownloadedFileDescriptor dfd){
			if(dfd.allocate(fileSize(dfd.getFilepath()))) fail();
			
			RandomAccessFile raf = null;
			try {
				raf = new RandomAccessFile(new File(dfd.getFilepath() + ".bck"), "r");
			} catch (FileNotFoundException e) {
				fail();
			}
			
			for(int i=0; i < dfd.getPartsCount() ; i++){
				byte[] data = new byte[DownloadedFileDescriptor.part_size];
				try {
					raf.seek(i * DownloadedFileDescriptor.part_size);
					raf.read(data);
				} catch (IOException e) {
					fail();
				}
				try {
					dfd.addPart(i, data);
				} catch (IOException e) {
					fail();
				}
			}
		}
		public void testPartsSaveFile0(){
			helpTestPartsSave(file0);
		}
		public void testPartsSaveFile1(){
			helpTestPartsSave(file1);
		}
		public void helpTestPartsSave(DownloadedFileDescriptor dfd){
			helpTestAddPart(dfd);
			dfd.saveBufferFile();
			if(!filesIsContentSame(dfd.getBufferFilepath(), dfd.getFilepath()))
				fail();
		}
		
		public void testBackupFile0(){
			helpTestBackup(file0);
		}
		public void testBackupFile1(){
			helpTestBackup(file1);
		}
		public void helpTestBackup(DownloadedFileDescriptor dfd){
			helpTestAddPart(dfd);
			
			if(dfd.getFileParts().size() != dfd.getPartsCount()) fail();
			
			dfd.backup();
			
			if(!fileExists(dfd.getFilePartsFilepath())) fail();
			if(fileSize(dfd.getFilePartsFilepath()) != dfd.getPartsCount() * 4) fail();
			
			//Load and compare arrays 
			DownloadedFileDescriptor dfd2 = new DownloadedFileDescriptor(dfd.getFilename(), dfd.getFolder(), dfd.getTargetChecksum());
			if(!dfd2.load()) fail();
			
			System.out.println(dfd2.getFileParts().toString());
			
			if(!dfd.getFileParts().equals(dfd2.getFileParts())) fail();
		}

		public void testCompletizeFile0(){
			helpTestCompletize(file0);
		}
		public void testCompletizeFile1(){
			helpTestCompletize(file1);
		}
		public void helpTestCompletize(DownloadedFileDescriptor dfd){
			helpTestAddPart(dfd);
			
			if(!dfd.completize()) fail();
			
			if(!filesIsContentSame(dfd.getFilepath(), dfd.getFilepath() + ".bck")) fail();
			//Check if it deleted the corresponding buffer files. 
			if(fileExists(dfd.getBufferFilepath())) fail(); 
			if(fileExists(dfd.getFilePartsFilepath())) fail();
		}
	
		/* -- from 30.04.2011
	   	public int getPartsCount(){}
	    
	    public double getProgress(){
	    	if(this.parts == null)
	    		return 0.0;
	    	return ((double) this.parts.size()) / ((double) this.getPartsCount());
	    }
	    //Tazup downloaded parts
	    public String getBufferFilepath(){}

		//Open for reading and writing, as with "rw", 
		//--and also require that every update to the file's content or 
		//--metadata be written synchronously to the underlying storage device.
	    protected boolean openBufferFile(){}
	    
	    //Allocates file on users file system (file_size in bytes)
	    //called when download starts.
	    //--also creates a descriptor file (which parts are missing)
	    //--the procedure is intelligent, nothing will be reallocated once allocated
	    public void allocate(long file_size) throws IOException{}
	    
	    private void allocateParts(int parts_count){}

	    public int getFirstMissingPart(){}
	    
	    public void addPart(int part_id, byte[] data) throws IOException{}
	    
	    //Stores the target file to target position
	    public boolean completize(){}
	    
	    //Stores the file parts file, saves the output file
	    // -- the try / catch blocks made this functions code quite unreadable...
	    public boolean backup(){}
	    

		//Open FilePartFiles and return the complement
	    protected boolean loadParts(){}
	    */
}
