package tests;

import java.util.Random;

import junit.framework.TestCase;
import types.*;
import auxiliary.*;

public class MessageTest extends TestCase {
	
	public void testTargetModule() {
		Message msg = new Message();
		
		msg.messageType = 0x00;
		if (! ModulType.PEER_MANAGEMENT.equals(msg.targetModule())) fail();
		
		msg.messageType = 0x01;
		if (! ModulType.PEER_MANAGEMENT.equals(msg.targetModule())) fail();
		
		msg.messageType = 0x02;
		if (! ModulType.SEARCH_ANSWER.equals(msg.targetModule())) fail();
		
		msg.messageType = 0x03;
		if (! ModulType.SEARCH.equals(msg.targetModule())) fail();
		
		msg.messageType = 0x04;
		if (! ModulType.UPLOAD.equals(msg.targetModule())) fail();
		
		msg.messageType = 0x05;
		if (! ModulType.DOWNLOAD.equals(msg.targetModule())) fail();
		
		msg.messageType = 0x06;
		if (! ModulType.DOWNLOAD.equals(msg.targetModule())) fail();
		
		msg.messageType = 0x07;
		if (! ModulType.UNKNOWN.equals(msg.targetModule())) fail();
	}
	
	public void testGetBytes_nullParams() {
		Message msg = new Message();
		if (msg.getBytes() != null) fail();
		
		msg.targetID = new PeerID(0);
		if (msg.getBytes() != null) fail();
		
		msg.sourceID = new PeerID(0);
		if (msg.getBytes() != null) fail();
		
		msg.identifier = new SequenceID(0);
		if (msg.getBytes() != null) fail();
		
		msg.data = new byte[0];
		if (msg.getBytes() == null) fail();
	}
	
	public void testGetBytes_validResult() {
		Message msg = new Message();
		msg.targetID = new PeerID(1);
		msg.sourceID = new PeerID(2);
		msg.identifier = new SequenceID(3);
		msg.messageType = 0xf;
		msg.dataLength = 10;
		msg.data = new byte[32];
		for (int i=0; i<msg.data.length; i++) msg.data[i] = (byte)i;
		
		byte[] bytes = msg.getBytes();
		if (bytes.length != Message.HEADER_SIZE + msg.data.length) fail();
		
		if (bytes[0] != 't' || bytes[1] != 'z' || bytes[2] != 'p') fail();
		if (bytes[3] != msg.messageType) fail();
		if (! msg.sourceID.equals(PeerID.peerIDfromBytes(bytes, 4))) fail();
		if (! msg.targetID.equals(PeerID.peerIDfromBytes(bytes, 8))) fail();
		if (! msg.identifier.equals(SequenceID.sequenceIDfromBytes(bytes, 12))) fail();
		if (msg.dataLength != Convert.bytesToInt(bytes, 16)) fail();
		for (int i=0; i<msg.data.length; i++)
			if (msg.data[i] != bytes[Message.HEADER_SIZE + i]) fail();
	}
	
	public void testMessageHeaderFromBytes_invalidInput() {
		Message msg;
		byte[] bytes = new byte[Message.HEADER_SIZE];
		
		msg = Message.messageHeaderFromBytes(null);
		if (msg != null) fail();
	
		msg = Message.messageHeaderFromBytes(new byte[1]);
		if (msg != null) fail();
		
		msg = Message.messageHeaderFromBytes(bytes);
		if (msg != null) fail();
		
		bytes[0] = 't'; bytes[1] = 'z'; bytes[2] = 'p';
		msg = Message.messageHeaderFromBytes(bytes);
		if (msg == null) fail();
	}
	
	public void testMessageHeaderFromBytes_validResult() {
		Random rnd = new Random(0);
		byte[] bytes = new byte[Message.HEADER_SIZE];
		
		rnd.nextBytes(bytes);
		bytes[0] = 't'; bytes[1] = 'z'; bytes[2] = 'p';
		Message msg = Message.messageHeaderFromBytes(bytes);
		
		if (msg ==  null) fail();
		if (msg.messageType != bytes[3]) fail();
		if (!PeerID.peerIDfromBytes(bytes, 4).equals(msg.sourceID)) fail();
		if (!PeerID.peerIDfromBytes(bytes, 8).equals(msg.targetID)) fail();
		if (!SequenceID.sequenceIDfromBytes(bytes, 12).equals(msg.identifier)) fail();
		if (msg.dataLength != Convert.bytesToInt(bytes, 16)) fail();
	}
	
	public void testMessageHeaderFormMessage_invalidInput() {
		Message msgh = Message.messageHeaderFromMessage(null);
		if (msgh != null) fail();
	}
	
	public void testMessageHeaderFromMessage_validResult() {
		Random rnd = new Random(0);
		byte[] bytes = new byte[Message.HEADER_SIZE];
		rnd.nextBytes(bytes);
		bytes[0] = 't'; bytes[1] = 'z'; bytes[2] = 'p';
		Message msg = Message.messageHeaderFromBytes(bytes);

		Message msgh = Message.messageHeaderFromMessage(msg);
		if (msgh == null) fail();
		if (msgh == msg) fail();

		if (msgh.messageType != msg.messageType) fail();
		if (!msg.sourceID.equals(msgh.sourceID)) fail();
		if (!msg.targetID.equals(msgh.targetID)) fail();
		if (!msg.identifier.equals(msgh.identifier)) fail();
		if (msgh.dataLength != msg.dataLength) fail();
	}
	
	public void testMessageFromBytes_invalidInput() {
		Message msg;
		byte[] bytes = new byte[Message.HEADER_SIZE+100];
		
		msg = Message.messageFromBytes(null, 1000);
		if (msg != null) fail();
	
		msg = Message.messageFromBytes(new byte[1], 1000);
		if (msg != null) fail();
		
		msg = Message.messageFromBytes(bytes, 1000);
		if (msg != null) fail();
		
		bytes[0] = 't'; bytes[1] = 'z'; bytes[2] = 'p';
		byte[] dataLen = Convert.intToBytes(1000);
		for (int i=0; i<4; i++) bytes[16+i] = dataLen[i];
		msg = Message.messageFromBytes(bytes, 50);
		if (msg != null) fail();
		
		dataLen = Convert.intToBytes(bytes.length-Message.HEADER_SIZE);
		for (int i=0; i<4; i++) bytes[16+i] = dataLen[i];
		msg = Message.messageFromBytes(bytes, bytes.length);
		if (msg == null) fail();
	}
	
	public void testMessageFromBytes_validResult() {
		Random rnd = new Random(0);
		byte[] bytes = new byte[Message.HEADER_SIZE + rnd.nextInt(1000)];
		rnd.nextBytes(bytes);
		bytes[0] = 't'; bytes[1] = 'z'; bytes[2] = 'p';
		byte[] dataLen = Convert.intToBytes(bytes.length-Message.HEADER_SIZE);
		for (int i=0; i<4; i++) bytes[16+i] = dataLen[i];
		Message msg = Message.messageFromBytes(bytes, bytes.length);
		
		if (msg ==  null) fail();
		if (msg.messageType != bytes[3]) fail();
		if (!PeerID.peerIDfromBytes(bytes, 4).equals(msg.sourceID)) fail();
		if (!PeerID.peerIDfromBytes(bytes, 8).equals(msg.targetID)) fail();
		if (!SequenceID.sequenceIDfromBytes(bytes, 12).equals(msg.identifier)) fail();
		if (msg.dataLength != Convert.bytesToInt(bytes, 16)) fail();
		for (int i=0; i<msg.dataLength; i++)
			if (bytes[Message.HEADER_SIZE + i] != msg.data[i]) fail();
	}
}
