package tests;

import java.io.File;

import auxiliary.SHA1;
import junit.framework.TestCase;
import types.*;

public class SearchResultTest extends TestCase {
	
	public void testMessageParsing() {
		byte[] hash = new byte[SHA1.SHA1_LENGTH];
		for (byte i=0; i<SHA1.SHA1_LENGTH; i++) hash[i] = i;
		SearchResult sr = new SearchResult(new PeerID(0), "filename", 12345678, new SHA1(hash));
		
		Message msg = new Message();
		msg.data = sr.getBytes();
		msg.messageType = 0x03;
		msg.sourceID = new PeerID(0);
		SearchResult nsr = SearchResult.fromMessage(msg);
		
		if (!sr.equals(nsr)) fail();
	}
	
	public void testFromSharedFileDescriptor() {
		try {
			File f = File.createTempFile("test", null);
			SharedFileDescriptor sfd = new SharedFileDescriptor(f.getName(), f.getParent(), SHA1.getFileHash(f.getPath()));
			SearchResult sr = SearchResult.fromSharedFileDescriptor(sfd);
			
			if (sr == null) fail();
			if (!f.getName().equals(sr.filename())) fail();
			if (f.length() != sr.filesize()) fail();
			if (!sfd.getTargetChecksum().equals(sr.checksum())) fail();
			
		}
		catch (Exception e) {
			e.printStackTrace();
			fail();
		}
	}
}
