package tests;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.TreeSet;
import java.util.concurrent.TimeUnit;

import auxiliary.SHA1;

import junit.framework.TestCase;

import testmodules.*;
import types.*;
import modules.*;


public class SearchingTest extends TestCase {

	private long TIMEOUT = 100;
	
	DataStorage ds;
	Communication comm;
	
	public SearchingTest() {
		ds = DataStorageTestingSubclass.getInstance();
		comm =  ds.getCommunicationInstance();
	}
	
	//---------------
	
	public void testSearch_querySent() {
		SearchModul sm = ds.getSearchInstance();
		SearchQuery sq = makeQuery("blabla");
		sm.search(sq);

		try {
			Message msg = comm.receiveMessage(ModulType.SEARCH_ANSWER, TIMEOUT, TimeUnit.MILLISECONDS);
			if (msg == null) fail();
		} catch (InterruptedException e) { fail(); }
	}
	
	public void testSearch_gotAnswer() {
		try {
			System.out.println("--------");
			addSharedFile("test");
			addSharedFile("test");
		
			SearchModul sm = new SearchModul();
			SearchAnswerModul sa = new SearchAnswerModul();
		
			Thread search = new Thread(sm);
			Thread answer = new Thread(sa);
			search.start();
			answer.start();
		
			Thread.sleep(TIMEOUT);
			sm.search(makeQuery("test"));
			Thread.sleep(TIMEOUT);
			sm.exit();
			sa.exit();
			
			if (ds.getSearchResults().length != 2) fail();
			
		} catch (InterruptedException e) {}
	}
	
	//----------------
	private SearchQuery makeQuery(String keyword) {
		SearchQuery sq = new SearchQuery();
		
		sq.keywords = new TreeSet<String>();
		sq.keywords.add(keyword);
		
		return sq;
	}
	
	private void addSharedFile(String prefix) {
		try {
			File f = File.createTempFile(prefix, null);
			FileOutputStream fs = new FileOutputStream(f);
			fs.write(f.getAbsolutePath().getBytes());
			SharedFilesDatabase db = ds.SharedFiles();
			SharedFileDescriptor sfd;
			sfd = new SharedFileDescriptor(f.getName(), f.getParent(), SHA1.getFileHash(f.getPath()));
			db.add(sfd);
		} catch (Exception e) { 
			e.printStackTrace();
			fail(); 
		}
	}
}
