package tests;

import java.io.IOException;
import java.io.OutputStream;
import java.net.InetAddress;
import java.net.ServerSocket;
import java.net.Socket;
import java.util.HashMap;
import java.util.concurrent.ArrayBlockingQueue;
import java.util.concurrent.TimeUnit;

import junit.framework.TestCase;

import modules.*;
import types.*;


public class TCPClientListenerTest extends TestCase {
	private int TIMEOUT = 50;	
	private int recvPort = 4747;
	private StreamID sid = new StreamID(47);
	private TCPClientListener listener;
	private ServerSocket server;
	private DataStorage ds = new DataStorage();
	private HashMap<ModulType, ArrayBlockingQueue<Message> > ms = new HashMap<ModulType, ArrayBlockingQueue<Message>>();

	public TCPClientListenerTest() {
		for (ModulType type: ModulType.values())
			ms.put(type, new ArrayBlockingQueue<Message>(100));
	}
	//---------
	
	public void testExit() {		
		try {			
			listener = new TCPClientListener(ds, ms, null,  new Socket(), sid);
			Thread t = new Thread(listener);
			t.start();
			
			Thread.sleep(TIMEOUT);
			listener.exit();
			Thread.sleep(TIMEOUT);
			
			if (! listener.tcpSocket().isClosed()) fail();
			if (t.isAlive()) fail();
		}
		catch (Exception e) {
			fail();
		}
	}
	// -------
	public void testRecieveMessage_shouldRecieve() {
		startListener();
		sendTCPMessage((byte)0x05, false, false, 1024);
		try {
			Message msg =  ms.get(ModulType.DOWNLOAD).poll(100, TimeUnit.MILLISECONDS);
			if (msg == null) safefail();
			if (msg.dataLength != 1024) safefail();
 		} catch (InterruptedException e) {
			safefail();
		}
		stopListener();
	}
	
	public void testRecieveMessage_shouldRecieveConnectionClosed() {
		startListener();
		sendTCPMessage((byte)0x05, false, false, 1024);
		try {
			Message msg =  ms.get(ModulType.DOWNLOAD).poll(100, TimeUnit.MILLISECONDS);
			if (msg == null) safefail();
			msg =  ms.get(ModulType.DOWNLOAD).poll(100, TimeUnit.MILLISECONDS);
			if (msg == null) safefail();
		} catch (InterruptedException e) {
			safefail();
		}
		stopListener();
	}
	//-----
	public void testValidMessageRecieved_sourceID() {
		startListener();
		sendTCPMessage((byte)0x05, false, false, 1024);
		try {
			Message msg =  ms.get(ModulType.DOWNLOAD).poll(100, TimeUnit.MILLISECONDS);
			if (!ds.myPeerID().equals(msg.sourceID)) safefail();
 		} catch (InterruptedException e) {
			safefail();
		}
		stopListener();
	}
	
	public void testValidMessageRecieved_targetID() {
		startListener();
		sendTCPMessage((byte)0x05, false, false, 1024);
		try {
			Message msg =  ms.get(ModulType.DOWNLOAD).poll(100, TimeUnit.MILLISECONDS);
			if (!ds.myPeerID().equals(msg.targetID)) safefail();
 		} catch (InterruptedException e) {
			safefail();
		}
		stopListener();
	}
	
	public void testValidMessageRecieved_remotePeer() {
		startListener();
		sendTCPMessage((byte)0x05, false, false, 1024);
		try {
			Message msg =  ms.get(ModulType.DOWNLOAD).poll(100, TimeUnit.MILLISECONDS);
			PeerInfo pi = msg.remotePeer;
			if (!ds.myPeerID().equals(pi.id())) safefail();
			if (!InetAddress.getLocalHost().equals(pi.ip())) safefail();
 		} catch (Exception e) {
			safefail();
		}
		stopListener();
	}
	
	//----- AUXILIARY
	private void startListener() {
		
		final class Listener implements Runnable {
			@Override
			public void run() {
				try {
					server = new ServerSocket(recvPort);
					Socket sock = server.accept();
					if (sock == null) safefail();
					listener = new TCPClientListener(ds, ms, null, sock, sid);
					Thread t = new Thread(listener);
					t.start();
				} catch (IOException e) {
					safefail();
				}	
			}
		}
		
		Thread t = new Thread(new Listener());
		t.start();
		try {
			Thread.sleep(TIMEOUT);
		} catch (InterruptedException e) {
			safefail();
		}
	}
	
	private void stopListener() {
		if (listener != null) listener.exit();
		try {
			if (server != null) server.close();
		} catch (IOException e) { fail(); }
	}
	
	private void sendTCPMessage(byte type, boolean invalidTarget, boolean invalidLength, int dataLen) {
		Message msg = new Message();
		msg.messageType = type;
		msg.sourceID = ds.myPeerID();
		msg.targetID = invalidTarget ? new PeerID(ds.myPeerID().value()+1) : ds.myPeerID();
		msg.identifier = new SequenceID(0);
		msg.dataLength = invalidLength ? dataLen + 10 : dataLen;
		msg.data = new byte[dataLen];
		
		byte[] bytes = msg.getBytes();
		
		try {
			Socket sock = new Socket(InetAddress.getLocalHost(), recvPort);
			OutputStream out = sock.getOutputStream();
			out.write(bytes);
			sock.close();
			
		} catch (Exception e) {
			e.printStackTrace();
			safefail();
		}
	}
	
	private void safefail() {
		stopListener();
		fail();
	}
}
